@@ -31,23 +31,9 @@ case "$os" in
 	plat_cflags="$plat_cflags -D_REENTRANT"
 	;;
 	Darwin)
+	plat_files="$plat_files ../src/compat/dummy.c"
 	plat_files="$plat_files ../src/compat/clearenv.c "
-	plat_files="$plat_files ../src/compat/dir_mutex.c "
-	plat_files="$plat_files ../src/compat/faccessat.c"
-	plat_files="$plat_files ../src/compat/fchmodat.c"
-	plat_files="$plat_files ../src/compat/fchownat.c"
-	plat_files="$plat_files ../src/compat/fdopendir.c"
-	plat_files="$plat_files ../src/compat/fstatat.c"
-	plat_files="$plat_files ../src/compat/mkdirat.c"
-	plat_files="$plat_files ../src/compat/openat.c"
-	plat_files="$plat_files ../src/compat/readlinkat.c"
-	plat_files="$plat_files ../src/compat/renameat.c"
-	plat_files="$plat_files ../src/compat/symlinkat.c"
-	plat_files="$plat_files ../src/compat/unlinkat.c"
-	plat_files="$plat_files ../src/compat/utimensat.c"
 	plat_cflags="$plat_cflags -include ../src/compat/macosx.h"
-	plat_cflags="$plat_cflags -DAT_SYMLINK_NOFOLLOW=0x100"
-	plat_cflags="$plat_cflags -DAT_REMOVEDIR=0x200"
 	CC=clang
 	;;
 	FreeBSD)
