@@ -2,7 +2,7 @@
  *
  * tup - A file-based build system
  *
- * Copyright (C) 2010-2018  Mike Shal <marfey@gmail.com>
+ * Copyright (C) 2010-2020  Mike Shal <marfey@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2 as
@@ -25,7 +25,7 @@
 
 extern pthread_mutex_t dir_mutex;
 
-void dir_mutex_lock(int dfd);
+int dir_mutex_lock(int dfd);
 void dir_mutex_unlock(void);
 
 #endif
