@@ -2,7 +2,7 @@
  *
  * tup - A file-based build system
  *
- * Copyright (C) 2010-2018  Mike Shal <marfey@gmail.com>
+ * Copyright (C) 2010-2020  Mike Shal <marfey@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2 as
@@ -29,7 +29,11 @@ int fstatat(int dirfd, const char *pathname, struct stat *buf, int flags)
 {
 	int rc;
 
-	dir_mutex_lock(dirfd);
+	rc = dir_mutex_lock(dirfd);
+	if(rc < 0) {
+		printf("fstatat[%s]: %i\n", pathname, rc);
+		return rc;
+	}
 	if(flags & AT_SYMLINK_NOFOLLOW) {
 		rc = lstat(pathname, buf);
 	} else {
