@@ -2,7 +2,7 @@
  *
  * tup - A file-based build system
  *
- * Copyright (C) 2012-2018  Mike Shal <marfey@gmail.com>
+ * Copyright (C) 2012-2020  Mike Shal <marfey@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2 as
@@ -24,6 +24,7 @@
  * the delete-on-close semantics of tmpfile() by setting the appropriate flag in
  * CreateFile().
  */
+#include <windows.h>
 #include <stdio.h>
 #include <errno.h>
 #include "tup/config.h"
@@ -38,9 +39,12 @@ FILE *__wrap_tmpfile(void)
 	char filename[64];
 	wchar_t wfilename[64];
 	FILE *f = NULL;
+	int rc;
 	HANDLE h;
 
-	dir_mutex_lock(tup_top_fd());
+	rc = dir_mutex_lock(tup_top_fd());
+	if(rc < 0)
+		return NULL;
 
 	if(mkdir(".tup/tmp", 0777) < 0) {
 		if(errno != EEXIST) {
