@@ -3,7 +3,7 @@
  * tup - A file-based build system
  *
  * Copyright (C) 2010  James McKaskill
- * Copyright (C) 2010-2018  Mike Shal <marfey@gmail.com>
+ * Copyright (C) 2010-2020  Mike Shal <marfey@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2 as
@@ -63,6 +63,15 @@
 #define __reserved
 #endif
 
+#ifndef FileRenameInformationEx
+/* This doesn't appear in MinGW (or at least the version I have), so we have to
+ * define it manually. The cygwin version of mv can use FileRenameInformationEx
+ * in certain circumstances, so we also have to check for that in
+ * NtSetInformationFile
+ */
+#define FileRenameInformationEx 65
+#endif
+
 typedef HFILE (WINAPI *OpenFile_t)(
     __in    LPCSTR lpFileName,
     __inout LPOFSTRUCT lpReOpenBuff,
@@ -492,7 +501,9 @@ NTSTATUS WINAPI NtCreateFile_hook(
 
 	DEBUG_HOOK("NtCreateFile[%08x] '%.*ls': %x, %x, %x\n", rc, uni->Length/2, uni->Buffer, ShareAccess, DesiredAccess, CreateOptions);
 
-	if (rc == STATUS_SUCCESS && DesiredAccess & TUP_CREATE_WRITE_FLAGS) {
+	if (rc == STATUS_SUCCESS &&
+	    (DesiredAccess & TUP_CREATE_WRITE_FLAGS) &&
+	    !(CreateOptions & FILE_DIRECTORY_FILE)) {
 		handle_file_w(uni->Buffer, uni->Length/2, NULL, ACCESS_WRITE);
 	} else {
 		handle_file_w(uni->Buffer, uni->Length/2, NULL, ACCESS_READ);
@@ -728,22 +739,21 @@ NTSTATUS WINAPI NtSetInformationFile_hook(
 	wchar_t widepath[WIDE_PATH_MAX];
 	wchar_t destpath[WIDE_PATH_MAX];
 	NTSTATUS rc;
-	DWORD len;
-	FILE_RENAME_INFORMATION *info = FileInformation;
+	DWORD len = 0;
 	int failed = 0;
 
-	if(FileInformationClass == FileRenameInformation) {
+	/* We need to get the old path before calling NtSetInformationFile in
+	 * case of a rename, in which case that information is lost.
+	 */
+	if(FileInformationClass == FileRenameInformation ||
+	   FileInformationClass == FileRenameInformationEx ||
+	   FileInformationClass == FileDispositionInformation) {
 		len = GetFinalPathNameByHandleW(FileHandle, widepath, WIDE_PATH_MAX, FILE_NAME_NORMALIZED);
 		if(len == 0) {
 			/* Failed to get path for some reason */
 			DEBUG_HOOK("NtSetInformationFile Error - failed to GetFinalPathNameByHandle\n");
 			failed = 1;
 		}
-		if(len + 1 + info->FileNameLength/2 + 1 > WIDE_PATH_MAX) {
-			/* Path too large. */
-			DEBUG_HOOK("NtSetInformationFile Error - path too long (%i, %i)\n", len, info->FileNameLength);
-			failed = 1;
-		}
 	}
 
 	rc = NtSetInformationFile_orig(FileHandle,
@@ -753,23 +763,32 @@ NTSTATUS WINAPI NtSetInformationFile_hook(
 				       FileInformationClass);
 	if(rc != STATUS_SUCCESS || failed)
 		return rc;
-
-	/* We're only checking this to see if a file gets renamed via this call. */
-	if(FileInformationClass != FileRenameInformation)
-		return rc;
-
-	if(info->FileNameLength / 2 >= WIDE_PATH_MAX) {
-		DEBUG_HOOK("NtSetInformationFile error - new path is too long.\n");
-		return rc;
-	}
 	DWORD save_error = GetLastError();
 
-	wcsncpy(destpath, info->FileName, info->FileNameLength);
-	destpath[info->FileNameLength / 2] = 0;
+	if(FileInformationClass == FileRenameInformation || FileInformationClass == FileRenameInformationEx) {
+		FILE_RENAME_INFORMATION *info = FileInformation;
+		if(info->FileNameLength / 2 >= WIDE_PATH_MAX) {
+			DEBUG_HOOK("NtSetInformationFile error - new path is too long.\n");
+			goto out_exit;
+		}
 
-	DEBUG_HOOK("NtSetInformationFile: rename '%S' -> '%S'\n", widepath, destpath);
-	handle_file_w(widepath, -1, destpath, ACCESS_RENAME);
+		wcsncpy(destpath, info->FileName, info->FileNameLength);
+		destpath[info->FileNameLength / 2] = 0;
 
+		DEBUG_HOOK("NtSetInformationFile[%i]: rename '%S' -> '%S'\n", FileInformationClass, widepath, destpath);
+		handle_file_w(widepath, len, destpath, ACCESS_RENAME);
+	} else if(FileInformationClass == FileDispositionInformation) {
+		FILE_DISPOSITION_INFORMATION *info = FileInformation;
+		if(info->DoDeleteFile) {
+			DEBUG_HOOK("NtSetInformationFile[%i]: delete on close '%S'\n", FileInformationClass, widepath);
+			handle_file_w(widepath, len, NULL, ACCESS_UNLINK);
+		} else {
+			DEBUG_HOOK("NtSetInformationFile[%i]: dont delete on close '%S'\n", FileInformationClass, widepath);
+			handle_file_w(widepath, len, NULL, ACCESS_WRITE);
+		}
+	}
+
+out_exit:
 	SetLastError(save_error);
 	return rc;
 }
@@ -1425,6 +1444,7 @@ int _access_hook(const char *pathname, int mode)
 
 int rename_hook(const char *oldpath, const char *newpath)
 {
+	DEBUG_HOOK("rename(%s, %s)\n", oldpath, newpath);
 	handle_file(oldpath, newpath, ACCESS_RENAME);
 	return rename_orig(oldpath, newpath);
 }
@@ -1744,7 +1764,7 @@ static void mhandle_file(const char* file, const char* file2, enum access_type a
 		dest += e->len;
 		*(dest++) = '\0';
 		*(dest++) = '\0';
-		DEBUG_HOOK("WRITE EVENT %s: '%s' '%s'\n", access_type_name[at], ((char*)e) + sizeof(*e), ((char*)e) + sizeof(*e) + e->len + 1);
+		DEBUG_HOOK("SEND EVENT %s: '%s' '%s'\n", access_type_name[at], ((char*)e) + sizeof(*e), ((char*)e) + sizeof(*e) + e->len + 1);
 		ret = writef((char*) e, dest - (char*) e);
 		DEBUG_HOOK("writef %d\n", ret);
 		if(ret) {}
@@ -1797,7 +1817,7 @@ static void mhandle_file_w(const wchar_t* file, int filelen, const wchar_t* file
 	DEBUG_HOOK("Canonicalize2: '%ls' -> '%s' len2=%i\n", file2, dest, e->len2);
 	dest += e->len2 + 1;
 
-	DEBUG_HOOK("WRITE EVENT %s [%i, %i]: '%s' '%s'\n", access_type_name[at], e->len, e->len2, ((char*)e) + sizeof(*e), ((char*)e) + sizeof(*e) + e->len + 1);
+	DEBUG_HOOK("SEND EVENT %s [%i, %i]: '%s' '%s'\n", access_type_name[at], e->len, e->len2, ((char*)e) + sizeof(*e), ((char*)e) + sizeof(*e) + e->len + 1);
 
 	ret = writef(buf, dest - buf);
 	DEBUG_HOOK("writef [wide] %d\n", ret);
@@ -1943,7 +1963,7 @@ static void WINAPI remote_init( remote_thread_t *r )
 	if (!h)
 		return;
 
-	p = (tup_init_t) r->get_proc_address(h, r->func_name);
+	p = (tup_init_t)(void (*)(void)) r->get_proc_address(h, r->func_name);
 	if (!p)
 		return;
 
@@ -2240,7 +2260,7 @@ int tup_inject_dll(
 
 		unsigned char code[code_size];
 
-		memcpy( code, &remote_stub32, code_size );
+		memcpy( code, &remote_stub32, sizeof(remote_stub32) );
 
 		*(DWORD*)(code + 0x1) = ctx.Eip;											// Return addr
 		*(DWORD*)(code + 0x8) = (DWORD)((DWORD_PTR)remote_data + code_size);							// Arg (ptr to remote (TCB))
@@ -2269,8 +2289,8 @@ int tup_inject_dll(
 
 		memset(&remote, 0, sizeof(remote));
 		kernel32 = LoadLibraryA("kernel32.dll");
-		remote.load_library = (LoadLibraryA_t) GetProcAddress(kernel32, "LoadLibraryA");
-		remote.get_proc_address = (GetProcAddress_t) GetProcAddress(kernel32, "GetProcAddress");
+		remote.load_library = (LoadLibraryA_t)(void (*)(void)) GetProcAddress(kernel32, "LoadLibraryA");
+		remote.get_proc_address = (GetProcAddress_t)(void (*)(void)) GetProcAddress(kernel32, "GetProcAddress");
 		strcpy(remote.depfilename, depfilename);
 		strcpy(remote.vardict_file, vardict_file);
 		strcat(remote.execdir, execdir);
