@@ -2,7 +2,7 @@
  *
  * ldpreload - Shared library shim for file accesses.
  *
- * Copyright (c) 2008-2018  Mike Shal <marfey@gmail.com>
+ * Copyright (C) 2008-2020  Mike Shal <marfey@gmail.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -51,6 +51,7 @@ int __xstat64(int __ver, __const char *__filename,
 	      struct stat64 *__stat_buf);
 int __lxstat64(int vers, const char *path, struct stat64 *buf);
 char *__realpath_chk(const char *path, char *resolved_path, size_t resolvedlen);
+void _mcleanup(void);
 
 static char cwd[PATH_MAX];
 static int cwdlen = -1;
@@ -61,6 +62,7 @@ static int update_cwd(void);
 
 static int (*s_open)(const char *, int, ...);
 static int (*s_open64)(const char *, int, ...);
+static int (*s_openat)(int, const char *, int, ...);
 static FILE *(*s_fopen)(const char *, const char *);
 static FILE *(*s_fopen64)(const char *, const char *);
 static FILE *(*s_freopen)(const char *, const char *, FILE *);
@@ -72,6 +74,7 @@ static char *(*s_realpath)(const char *, char *);
 static char *(*s_realpath_chk)(const char *, char *, size_t);
 static int (*s_rename)(const char*, const char*);
 static int (*s_renameat)(int, const char*, int, const char*);
+static int (*s_renameat2)(int, const char*, int, const char*, unsigned int flags);
 static int (*s_mkstemp)(char *template);
 static int (*s_mkostemp)(char *template, int flags);
 static int (*s_remove)(const char *);
@@ -87,6 +90,7 @@ static int (*s_xstat)(int vers, const char *name, struct stat *buf);
 static int (*s_stat64)(const char *name, struct stat64 *buf);
 static int (*s_xstat64)(int vers, const char *name, struct stat64 *buf);
 static int (*s_lxstat64)(int vers, const char *path, struct stat64 *buf);
+static void (*s_mcleanup)(void);
 
 #define WRAP(ptr, name) \
 	if(!ptr) { \
@@ -215,6 +219,31 @@ int open64(const char *pathname, int flags, ...)
 	return rc;
 }
 
+int openat(int dfd, const char *pathname, int flags, ...)
+{
+	int rc;
+	mode_t mode = 0;
+
+	WRAP(s_openat, "openat");
+	if(flags & O_CREAT) {
+		va_list ap;
+		va_start(ap, flags);
+		mode = va_arg(ap, int);
+		va_end(ap);
+	}
+	rc = s_openat(dfd, pathname, flags, mode);
+	if(rc >= 0) {
+		int at = ACCESS_READ;
+
+		if(flags&O_WRONLY || flags&O_RDWR)
+			at = ACCESS_WRITE;
+		handle_file_dirfd(dfd, pathname, "", at);
+	} else {
+		handle_file_dirfd(dfd, pathname, "", ACCESS_READ);
+	}
+	return rc;
+}
+
 FILE *fopen(const char *path, const char *mode)
 {
 	FILE *f;
@@ -362,6 +391,26 @@ int renameat(int oldfd, const char *old, int newfd, const char *new)
 	return rc;
 }
 
+int renameat2(int oldfd, const char *old, int newfd, const char *new, unsigned int flags)
+{
+	int rc;
+
+	/* This shouldn't be too hard to implement, but at the moment we only
+	 * need renameat() with AT_FDCWD for 'ln -s' on arch and fedora.
+	 */
+	if(oldfd != AT_FDCWD || newfd != AT_FDCWD) {
+		fprintf(stderr, "tup error: renameat2() with fd != AT_FDCWD is not yet supported.\n");
+		errno = ENOSYS;
+		return -1;
+	}
+	WRAP(s_renameat2, "renameat2");
+	rc = s_renameat2(oldfd, old, newfd, new, flags);
+	if(rc == 0) {
+		handle_file(old, new, ACCESS_RENAME);
+	}
+	return rc;
+}
+
 int mkstemp(char *template)
 {
 	int rc;
@@ -545,6 +594,19 @@ int __lxstat64(int vers, const char *path, struct stat64 *buf)
 	return rc;
 }
 
+/* This function is called by glibc to write out the gmon.out file when
+ * programs are compiled with -pg. Unfortunately it does so using a direct
+ * syscall, so we miss the hook. The FUSE checker would catch gmon.out, which
+ * makes it hard to write an accurate Tupfile if FUSE sees it but ldpreload
+ * does not.
+ */
+void _mcleanup(void)
+{
+	WRAP(s_mcleanup, "_mcleanup");
+	handle_file("gmon.out", "", ACCESS_WRITE);
+	s_mcleanup();
+}
+
 static int write_all(int fd, const void *data, int size)
 {
 	if(write(fd, data, size) != size) {
