@@ -2,7 +2,7 @@
  *
  * tup - A file-based build system
  *
- * Copyright (C) 2008-2018  Mike Shal <marfey@gmail.com>
+ * Copyright (C) 2008-2020  Mike Shal <marfey@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2 as
@@ -47,7 +47,7 @@ int create_name_file(tupid_t dt, const char *file, time_t mtime,
 	struct tup_entry *dtent;
 	if(tup_entry_add(dt, &dtent) < 0)
 		return -1;
-	if(tup_db_node_insert_tent(dt, file, -1, TUP_NODE_FILE, mtime, -1, entry) < 0)
+	if(tup_db_node_insert_tent(dtent, file, -1, TUP_NODE_FILE, mtime, -1, entry) < 0)
 		return -1;
 	if(tup_db_add_create_list(dt) < 0)
 		return -1;
@@ -65,7 +65,10 @@ int create_name_file(tupid_t dt, const char *file, time_t mtime,
 tupid_t create_command_file(tupid_t dt, const char *cmd, const char *display, int displaylen, const char *flags, int flagslen)
 {
 	struct tup_entry *tent;
-	tent = tup_db_create_node_part_display(dt, cmd, -1, display, displaylen, flags, flagslen,
+	struct tup_entry *dtent;
+	if(tup_entry_add(dt, &dtent) < 0)
+		return -1;
+	tent = tup_db_create_node_part_display(dtent, cmd, -1, display, displaylen, flags, flagslen,
 					       TUP_NODE_CMD, -1, NULL);
 	if(tent)
 		return tent->tnode.tupid;
@@ -97,10 +100,13 @@ tupid_t tup_file_mod_mtime(tupid_t dt, const char *file, time_t mtime,
 			   int force, int ignore_generated, int *modified)
 {
 	struct tup_entry *tent;
+	struct tup_entry *dtent;
 	int new = 0;
 	int changed = 0;
 
-	if(tup_db_select_tent(dt, file, &tent) < 0)
+	if(tup_entry_add(dt, &dtent) < 0)
+		return -1;
+	if(tup_db_select_tent(dtent, file, &tent) < 0)
 		return -1;
 
 	if(!tent) {
@@ -134,7 +140,7 @@ tupid_t tup_file_mod_mtime(tupid_t dt, const char *file, time_t mtime,
 			log_debug_tent("Create(overwrite)", tent, ", oldtype=%i\n", tent->type);
 			if(tup_del_id_type(tent->tnode.tupid, tent->type, 1, NULL) < 0)
 				return -1;
-			if(tup_db_select_tent(dt, file, &tent) < 0)
+			if(tup_db_select_tent(dtent, file, &tent) < 0)
 				return -1;
 			if(!tent) {
 				if(create_name_file(dt, file, mtime, &tent) < 0)
@@ -215,11 +221,14 @@ static int check_rm_tup_config(struct tup_entry *tent, int *dont_delete)
 int tup_file_del(tupid_t dt, const char *file, int len, int *modified)
 {
 	struct tup_entry *tent;
+	struct tup_entry *dtent;
 
+	if(tup_entry_add(dt, &dtent) < 0)
+		return -1;
 	if(len < 0)
 		len = strlen(file);
 
-	if(tup_db_select_tent_part(dt, file, len, &tent) < 0)
+	if(tup_db_select_tent_part(dtent, file, len, &tent) < 0)
 		return -1;
 	if(!tent) {
 		/* If we are trying to delete a file that isn't in tup, that's
@@ -291,7 +300,7 @@ static struct tup_entry *get_variant_tent(struct tup_entry *srctent, struct vari
 	if(!parent_tent) {
 		return NULL;
 	}
-	if(tup_db_select_tent(parent_tent->tnode.tupid, srctent->name.s, &variant_tent) < 0) {
+	if(tup_db_select_tent(parent_tent, srctent->name.s, &variant_tent) < 0) {
 		return NULL;
 	}
 	return variant_tent;
@@ -374,7 +383,7 @@ int tup_del_id_type(tupid_t tupid, enum TUP_NODE_TYPE type, int force, int *modi
 			 * can be cleaned up as necessary (t8020).
 			 */
 			struct tup_entry *tuptent;
-			if(tup_db_select_tent(tupid, "Tupfile", &tuptent) < 0)
+			if(tup_db_select_tent(tent, "Tupfile", &tuptent) < 0)
 				return -1;
 			if(tuptent) {
 				if(tup_db_set_dependent_dir_flags(tuptent->tnode.tupid) < 0)
@@ -477,13 +486,17 @@ struct tup_entry *get_tent_dt(tupid_t dt, const char *path)
 {
 	struct path_element *pel = NULL;
 	struct tup_entry *tent;
+	struct tup_entry *dtent;
 
 	dt = find_dir_tupid_dt(dt, path, &pel, 0, 1);
 	if(dt <= 0)
 		return NULL;
 
+	if(tup_entry_add(dt, &dtent) < 0)
+		return NULL;
+
 	if(pel) {
-		if(tup_db_select_tent_part(dt, pel->path, pel->len, &tent) < 0)
+		if(tup_db_select_tent_part(dtent, pel->path, pel->len, &tent) < 0)
 			return NULL;
 		free(pel);
 		if(!tent)
@@ -507,6 +520,8 @@ tupid_t find_dir_tupid(const char *dir)
 	 */
 	if(strcmp(dir, "0") == 0)
 		return 0;
+	if(strcmp(dir, "/") == 0)
+		return slash_dt();
 	tent = get_tent_dt(dt, dir);
 	if(!tent)
 		return -1;
@@ -582,10 +597,9 @@ tupid_t find_dir_tupid_dt_pg(tupid_t dt, struct pel_group *pg,
 			}
 			tent = tent->parent;
 		} else {
-			tupid_t curdt;
+			struct tup_entry *curtent = tent;
 
-			curdt = tent->tnode.tupid;
-			if(tup_db_select_tent_part(curdt, pel->path, pel->len, &tent) < 0)
+			if(tup_db_select_tent_part(tent, pel->path, pel->len, &tent) < 0)
 				return -1;
 			if(tent) {
 				if(sotgv == SOTGV_CREATE_DIRS) {
@@ -604,6 +618,8 @@ tupid_t find_dir_tupid_dt_pg(tupid_t dt, struct pel_group *pg,
 				}
 			} else {
 				int type = TUP_NODE_GHOST;
+				time_t mtime = -1;
+
 				/* Secret of the ghost valley! */
 				if(sotgv == 0) {
 					return -1;
@@ -612,7 +628,12 @@ tupid_t find_dir_tupid_dt_pg(tupid_t dt, struct pel_group *pg,
 					type = TUP_NODE_GENERATED_DIR;
 				else if(sotgv == SOTGV_IGNORE_DIRS)
 					type = TUP_NODE_DIR;
-				if(tup_db_node_insert_tent(curdt, pel->path, pel->len, type, -1, -1, &tent) < 0)
+
+				if(full_deps && (pg->pg_flags & PG_OUTSIDE_TUP)) {
+					if(get_outside_tup_mtime(curtent, pel, &mtime) < 0)
+						return -1;
+				}
+				if(tup_db_node_insert_tent(curtent, pel->path, pel->len, type, mtime, -1, &tent) < 0)
 					return -1;
 			}
 		}
@@ -623,12 +644,61 @@ tupid_t find_dir_tupid_dt_pg(tupid_t dt, struct pel_group *pg,
 	return tent->tnode.tupid;
 }
 
+int get_outside_tup_mtime(struct tup_entry *parent, struct path_element *pel, time_t *mtime)
+{
+	int dfd;
+
+	dfd = tup_entry_open(parent);
+	if(dfd == -ENOENT || dfd == -ENOTDIR) {
+		*mtime = -1;
+	} else if(dfd < 0) {
+		perror("tup_entry_open");
+		fprintf(stderr, "tup error: Unable to open tup entry: ");
+		print_tup_entry(stderr, parent);
+		fprintf(stderr, "\n");
+		return -1;
+	} else {
+		struct stat buf;
+		char tmppath[PATH_MAX];
+
+		/* Pel's aren't nul-terminated, so unfortunately we have to
+		 * make a copy here.
+		 */
+		strncpy(tmppath, pel->path, pel->len);
+		tmppath[pel->len] = 0;
+		if(fstatat(dfd, tmppath, &buf, AT_SYMLINK_NOFOLLOW) < 0) {
+			if(errno != ENOENT && errno != ENOTDIR) {
+				perror("fstatat");
+				fprintf(stderr, "tup error: Unable to stat file: %.*s\n", pel->len, pel->path);
+				return -1;
+			}
+			*mtime = -1;
+		} else {
+			/* Ghost directories in the /-tree have mtimes set to
+			 * zero if they exist. This way we can distinguish
+			 * between a directory being created where there wasn't
+			 * one previously (t4064, t4205).
+			 */
+			if(S_ISDIR(buf.st_mode))
+				*mtime = EXTERNAL_DIRECTORY_MTIME;
+			else
+				*mtime = MTIME(buf);
+		}
+		if(close(dfd) < 0) {
+			perror("close(dfd)");
+			return -1;
+		}
+	}
+	return 0;
+}
+
 int gimme_tent(const char *name, struct tup_entry **entry)
 {
 	tupid_t dt;
+	struct tup_entry *dtent;
 	struct path_element *pel = NULL;
 
-	dt = find_dir_tupid_dt(DOT_DT, name, &pel, 0, 0);
+	dt = find_dir_tupid_dt(DOT_DT, name, &pel, 0, 1);
 	if(dt < 0)
 		return -1;
 	if(dt == 0) {
@@ -639,7 +709,9 @@ int gimme_tent(const char *name, struct tup_entry **entry)
 		*entry = tup_entry_get(dt);
 		return 0;
 	}
-	if(tup_db_select_tent_part(dt, pel->path, pel->len, entry) < 0)
+	if(tup_entry_add(dt, &dtent) < 0)
+		return -1;
+	if(tup_db_select_tent_part(dtent, pel->path, pel->len, entry) < 0)
 		return -1;
 	free(pel);
 	return 0;
@@ -647,8 +719,7 @@ int gimme_tent(const char *name, struct tup_entry **entry)
 
 static int ghost_to_file(struct tup_entry *tent)
 {
-	if(tent->ghost_list.le_prev != NULL)
-		tup_entry_del_ghost_list(tent);
+	tup_db_del_ghost_tree(tent);
 	if(tup_db_set_type(tent, TUP_NODE_FILE) < 0)
 		return -1;
 	/* Only add dirs, not generated dirs, to the create list. */
