@@ -2,7 +2,7 @@
  *
  * tup - A file-based build system
  *
- * Copyright (C) 2009-2018  Mike Shal <marfey@gmail.com>
+ * Copyright (C) 2009-2020  Mike Shal <marfey@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2 as
@@ -27,6 +27,7 @@
 #include "container.h"
 #include "variant.h"
 #include "estring.h"
+#include "tent_tree.h"
 #include <stdlib.h>
 #include <string.h>
 #include <fcntl.h>
@@ -37,10 +38,8 @@
 #include <sys/stat.h>
 
 static struct tupid_entries tup_root = RB_INITIALIZER(&tup_root);
-static int list_out = 0;
-static struct tup_entry_head entry_list;
 static int do_verbose = 0;
-static pthread_mutex_t entry_openat_mutex;
+static pthread_mutex_t entry_openat_mutex = PTHREAD_MUTEX_INITIALIZER;
 
 static struct tup_entry *new_entry(tupid_t tupid, tupid_t dt,
 				   const char *name, int len,
@@ -51,20 +50,11 @@ static int rm_entry(tupid_t tupid, int safe);
 static int resolve_parent(struct tup_entry *tent);
 static int change_name(struct tup_entry *tent, const char *new_name);
 
-int tup_entry_init(void)
-{
-	if(pthread_mutex_init(&entry_openat_mutex, NULL) != 0) {
-		perror("pthread_mutex_init");
-		return -1;
-	}
-	return 0;
-}
-
 int tup_entry_add(tupid_t tupid, struct tup_entry **dest)
 {
 	struct tup_entry *tent;
 
-	if(tupid <= 0) {
+	if(tupid < 0) {
 		fprintf(stderr, "tup error: Tupid is %lli in tup_entry_add()\n",
 			tupid);
 		return -1;
@@ -90,28 +80,22 @@ int tup_entry_add(tupid_t tupid, struct tup_entry **dest)
 	return 0;
 }
 
-int tup_entry_find_name_in_dir_dt(tupid_t dt, const char *name, int len,
+int tup_entry_find_name_in_dir_dt(struct tup_entry *dtent, const char *name, int len,
 				  struct tup_entry **dest)
 {
-	struct tup_entry *parent;
-
 	if(len < 0)
 		len = strlen(name);
 
-	if(dt == 0) {
+	if(dtent->tnode.tupid == 0) {
 		if(strncmp(name, ".", len) == 0) {
 			*dest = tup_entry_find(DOT_DT);
 			return 0;
 		}
-		fprintf(stderr, "tup error: entry '%.*s' shouldn't have dt == 0\n", len, name);
+		fprintf(stderr, "tup error: entry '%.*s' shouldn't have dtent == NULL\n", len, name);
 		return -1;
 	}
 
-	if(tup_entry_add(dt, &parent) < 0) {
-		fprintf(stderr, "tup error: Unable to find parent entry [%lli] for node '%.*s'\n", dt, len, name);
-		return -1;
-	}
-	return tup_entry_find_name_in_dir(parent, name, len, dest);
+	return tup_entry_find_name_in_dir(dtent, name, len, dest);
 }
 
 int tup_entry_find_name_in_dir(struct tup_entry *tent, const char *name, int len,
@@ -158,13 +142,7 @@ static int rm_entry(tupid_t tupid, int safe)
 		}
 	}
 
-	if(tent->list.le_prev != NULL) {
-		fprintf(stderr, "tup internal error: tup_entry_rm called on tupid %lli, which is in the entry list [%lli:%s]\n", tupid, tent->dt, tent->name.s);
-		return -1;
-	}
-	if(tent->ghost_list.le_prev != NULL) {
-		LIST_REMOVE(tent, ghost_list);
-	}
+	tup_db_del_ghost_tree(tent);
 
 	tupid_tree_rm(&tup_root, &tent->tnode);
 	if(tent->parent) {
@@ -173,8 +151,12 @@ static int rm_entry(tupid_t tupid, int safe)
 	if(tent->re) {
 		pcre_free(tent->re);
 	}
-	free_tupid_tree(&tent->stickies);
-	free_tupid_tree(&tent->group_stickies);
+	free_tent_tree(&tent->stickies);
+	free_tent_tree(&tent->group_stickies);
+	if(tent->refcount != 0) {
+		fprintf(stderr, "tup internal error: tup_entry_rm called on tupid %lli, which still has refcount=%i\n", tupid, tent->refcount);
+		return -1;
+	}
 	free(tent->name.s);
 	free(tent->display);
 	free(tent->flags);
@@ -270,14 +252,14 @@ int snprint_tup_entry(char *dest, int len, struct tup_entry *tent)
 	return rc;
 }
 
-int tup_entry_add_to_dir(tupid_t dt, tupid_t tupid, const char *name, int len,
+int tup_entry_add_to_dir(struct tup_entry *dtent, tupid_t tupid, const char *name, int len,
 			 const char *display, int displaylen, const char *flags, int flagslen,
 			 enum TUP_NODE_TYPE type, time_t mtime, tupid_t srcid,
 			 struct tup_entry **dest)
 {
 	struct tup_entry *tent;
 
-	tent = new_entry(tupid, dt, name, len, display, displaylen, flags, flagslen, type, mtime, srcid);
+	tent = new_entry(tupid, dtent->tnode.tupid, name, len, display, displaylen, flags, flagslen, type, mtime, srcid);
 	if(!tent)
 		return -1;
 	if(resolve_parent(tent) < 0)
@@ -345,8 +327,8 @@ static int entry_openat_internal(int root_dfd, struct tup_entry *tent)
 		return -1;
 	}
 	if(newdfd < 0) {
-		if(errno == ENOENT)
-			return -ENOENT;
+		if(errno == ENOENT || errno == ENOTDIR)
+			return -errno;
 		perror(tent->name.s);
 		return -1;
 	}
@@ -371,6 +353,16 @@ int tup_entry_openat(int root_dfd, struct tup_entry *tent)
 	return rc;
 }
 
+void tup_entry_add_ref(struct tup_entry *tent)
+{
+	tent->refcount++;
+}
+
+void tup_entry_del_ref(struct tup_entry *tent)
+{
+	tent->refcount--;
+}
+
 struct variant *tup_entry_variant(struct tup_entry *tent)
 {
 	/* The variant field isn't set when we initially create tup_entrys, since
@@ -412,52 +404,6 @@ tupid_t tup_entry_vardt(struct tup_entry *tent)
 	return tup_entry_variant(tent)->tent->tnode.tupid;
 }
 
-static int create_dir(int dfd, struct tup_entry *tent)
-{
-	int curfd;
-	int newfd;
-
-	if(!tent->parent) {
-		return dup(dfd);
-	}
-
-	curfd = create_dir(dfd, tent->parent);
-	if(curfd < 0)
-		return -1;
-
-	if(mkdirat(curfd, tent->name.s, 0777) < 0) {
-		if(errno != EEXIST) {
-			perror(tent->name.s);
-			fprintf(stderr, "tup error: Unable to create sub-directory in the build tree.\n");
-			return -1;
-		}
-	}
-	newfd = openat(curfd, tent->name.s, O_RDONLY | O_CLOEXEC);
-	if(newfd < 0) {
-		perror(tent->name.s);
-		fprintf(stderr, "tup error: Unable to open newly-created sub-directory in the build tree.\n");
-		return -1;
-	}
-	if(close(curfd) < 0) {
-		perror("close(curfd)");
-		return -1;
-	}
-	return newfd;
-}
-
-int tup_entry_create_dirs(int root_dfd, struct tup_entry *tent)
-{
-	int newfd;
-	newfd = create_dir(root_dfd, tent);
-	if(newfd < 0)
-		return -1;
-	if(close(newfd) < 0) {
-		perror("close(newfd)");
-		return -1;
-	}
-	return 0;
-}
-
 static int set_string(char **dest, int *destlen, const char *src, int srclen)
 {
 	if(src) {
@@ -506,8 +452,6 @@ static struct tup_entry *new_entry(tupid_t tupid, tupid_t dt,
 	}
 
 	tent->tnode.tupid = tupid;
-	tent->list.le_prev = NULL;
-	tent->ghost_list.le_prev = NULL;
 	tent->dt = dt;
 	tent->parent = NULL;
 	tent->type = type;
@@ -518,6 +462,7 @@ static struct tup_entry *new_entry(tupid_t tupid, tupid_t dt,
 	RB_INIT(&tent->group_stickies);
 	tent->retrieved_stickies = 0;
 	tent->incoming = NULL;
+	tent->refcount = 0;
 	if(set_string(&tent->name.s, &tent->name.len, name, len) < 0)
 		return NULL;
 	if(set_string(&tent->display, &tent->displaylen, display, displaylen) < 0)
@@ -594,6 +539,20 @@ int tup_entry_change_flags(struct tup_entry *tent, const char *flags, int flagsl
 
 int tup_entry_clear(void)
 {
+	/* First delete all stickies & group_stickies. Even though these are
+	 * freed in rm_entry(), the refcount check would fail since we don't
+	 * delete nodes in any particular order. And unlike the entries tree
+	 * for directory entries, we don't have a way to remove a node from all
+	 * places where it might point to as a sticky link when it gets
+	 * deleted.
+	 */
+	struct tupid_tree *tt;
+	RB_FOREACH(tt, tupid_entries, &tup_root) {
+		struct tup_entry *tent = container_of(tt, struct tup_entry, tnode);
+		free_tent_tree(&tent->stickies);
+		free_tent_tree(&tent->group_stickies);
+	}
+
 	/* The rm_entry with safe=1 will only remove the node if all of the
 	 * children nodes are gone. Rather than try to smartly remove things
 	 * in the correct order, the outer loop will just keep going until
@@ -605,7 +564,6 @@ int tup_entry_clear(void)
 	 * upgrades.
 	 */
 	while(!RB_EMPTY(&tup_root)) {
-		struct tupid_tree *tt;
 		struct tupid_tree *tmp;
 
 		RB_FOREACH_SAFE(tt, tupid_entries, &tup_root, tmp) {
@@ -616,81 +574,17 @@ int tup_entry_clear(void)
 	return 0;
 }
 
-struct tup_entry_head *tup_entry_get_list(void)
-{
-	if(list_out) {
-		fprintf(stderr, "tup internal error: entry list is already out\n");
-		exit(1);
-	}
-	list_out = 1;
-	LIST_INIT(&entry_list);
-	return &entry_list;
-}
-
-void tup_entry_release_list(void)
-{
-	if(!list_out) {
-		fprintf(stderr, "tup internal error: entry list isn't out\n");
-		exit(1);
-	}
-	while(!LIST_EMPTY(&entry_list)) {
-		struct tup_entry *tent = LIST_FIRST(&entry_list);
-		tup_entry_list_del(tent);
-	}
-	list_out = 0;
-}
-
-void tup_entry_list_add(struct tup_entry *tent, struct tup_entry_head *head)
-{
-	if(!list_out) {
-		fprintf(stderr, "tup internal error: tup_entry_list_add called without the list\n");
-		exit(1);
-	}
-	if(tent->list.le_prev == NULL) {
-		LIST_INSERT_HEAD(head, tent, list);
-	}
-}
-
-void tup_entry_list_del(struct tup_entry *tent)
-{
-	if(!list_out) {
-		fprintf(stderr, "tup internal error: tup_entry_list_del called without the list\n");
-		exit(1);
-	}
-	LIST_REMOVE(tent, list);
-	tent->list.le_prev = NULL;
-}
-
-int tup_entry_in_list(struct tup_entry *tent)
-{
-	return !(tent->list.le_prev == NULL);
-}
-
-void tup_entry_add_ghost_list(struct tup_entry *tent, struct tup_entry_head *head)
+int tup_entry_add_ghost_tree(struct tup_entry *tent, struct tent_entries *root)
 {
 	if(tent->type == TUP_NODE_GHOST || tent->type == TUP_NODE_GROUP ||
 	   tent->type == TUP_NODE_GENERATED_DIR) {
-		/* It is fine if the ghost is already in the list - just make
-		 * sure we don't try to add it twice.
-		 */
-		if(tent->ghost_list.le_prev == NULL) {
-			LIST_INSERT_HEAD(head, tent, ghost_list);
-		}
+		if(tent_tree_add_dup(root, tent) < 0)
+			return -1;
 	}
-}
-
-int tup_entry_del_ghost_list(struct tup_entry *tent)
-{
-	if(tent->ghost_list.le_prev == NULL) {
-		fprintf(stderr, "tup internal error: ghost_list.next is NULL in tup_entry_del_ghost_list %lli [%lli:%s]\n", tent->tnode.tupid, tent->dt, tent->name.s);
-		return -1;
-	}
-	LIST_REMOVE(tent, ghost_list);
-	tent->ghost_list.le_prev = NULL;
 	return 0;
 }
 
-int tup_entry_debug_add_all_ghosts(struct tup_entry_head *head)
+int tup_entry_debug_add_all_ghosts(struct tent_entries *root)
 {
 	struct tupid_tree *tt;
 
@@ -698,7 +592,8 @@ int tup_entry_debug_add_all_ghosts(struct tup_entry_head *head)
 		struct tup_entry *tent;
 
 		tent = container_of(tt, struct tup_entry, tnode);
-		tup_entry_add_ghost_list(tent, head);
+		if(tup_entry_add_ghost_tree(tent, root) < 0)
+			return -1;
 	}
 	return 0;
 }
@@ -750,21 +645,6 @@ void dump_tup_entry(void)
 	}
 }
 
-void del_tent_list_entry(struct tent_list_head *head, struct tent_list *tlist)
-{
-	TAILQ_REMOVE(head, tlist, list);
-	free(tlist);
-}
-
-void free_tent_list(struct tent_list_head *head)
-{
-	struct tent_list *tlist;
-	while(!TAILQ_EMPTY(head)) {
-		tlist = TAILQ_FIRST(head);
-		del_tent_list_entry(head, tlist);
-	}
-}
-
 /* Note: when used with variants, this function will skip over the variant
  * root tent, meaning that the returned list of tents isn't a correct parent
  * list, but when converted to a path it will refer to the correct file.
@@ -772,7 +652,6 @@ void free_tent_list(struct tent_list_head *head)
 static int get_full_path_tents(tupid_t tupid, struct tent_list_head *head)
 {
 	struct tup_entry *tent;
-	struct tent_list *tlist;
 	struct variant *variant;
 
 	tent = tup_entry_get(tupid);
@@ -789,13 +668,8 @@ static int get_full_path_tents(tupid_t tupid, struct tent_list_head *head)
 				continue;
 			}
 		}
-		tlist = malloc(sizeof *tlist);
-		if(!tlist) {
-			perror("malloc");
+		if(tent_list_add_head(head, tent) < 0)
 			return -1;
-		}
-		tlist->tent = tent;
-		TAILQ_INSERT_HEAD(head, tlist, list);
 
 		tent = tent->parent;
 	}
@@ -810,26 +684,26 @@ int get_relative_dir(FILE *f, struct estring *e, tupid_t start, tupid_t end)
 	struct tent_list *endentry;
 	int first = 0;
 
-	TAILQ_INIT(&startlist);
-	TAILQ_INIT(&endlist);
+	tent_list_init(&startlist);
+	tent_list_init(&endlist);
 	if(get_full_path_tents(start, &startlist) < 0)
 		return -1;
 	if(get_full_path_tents(end, &endlist) < 0)
 		return -1;
 
-	while(!TAILQ_EMPTY(&startlist) && !TAILQ_EMPTY(&endlist)) {
-		startentry = TAILQ_FIRST(&startlist);
-		endentry = TAILQ_FIRST(&endlist);
+	while(!tent_list_empty(&startlist) && !tent_list_empty(&endlist)) {
+		startentry = tent_list_first(&startlist);
+		endentry = tent_list_first(&endlist);
 
 		if(startentry->tent == endentry->tent) {
-			del_tent_list_entry(&startlist, startentry);
-			del_tent_list_entry(&endlist, endentry);
+			tent_list_delete(&startlist, startentry);
+			tent_list_delete(&endlist, endentry);
 		} else {
 			break;
 		}
 	}
 
-	TAILQ_FOREACH(startentry, &startlist, list) {
+	tent_list_foreach(startentry, &startlist) {
 		if(!first) {
 			first = 1;
 		} else {
@@ -849,7 +723,7 @@ int get_relative_dir(FILE *f, struct estring *e, tupid_t start, tupid_t end)
 			if(estring_append(e, "..", 2) < 0)
 				return -1;
 	}
-	TAILQ_FOREACH(endentry, &endlist, list) {
+	tent_list_foreach(endentry, &endlist) {
 		if(!first) {
 			first = 1;
 		} else {
@@ -879,56 +753,25 @@ int get_relative_dir(FILE *f, struct estring *e, tupid_t start, tupid_t end)
 	return 0;
 }
 
-int exclusion_root_to_list(struct tupid_entries *root, struct re_entry_head *head)
+int exclusion_match(FILE *f, struct tent_entries *exclusion_root, const char *s, int *match)
 {
-	struct tupid_tree *tt;
-
-	RB_FOREACH(tt, tupid_entries, root) {
-		struct tup_entry *tent;
-		struct re_entry *re;
-
-		if(tup_entry_add(tt->tupid, &tent) < 0)
-			return -1;
-		re = malloc(sizeof *re);
-		if(!re) {
-			perror("malloc");
-			return -1;
-		}
-		re->re = tent->re;
-		re->s = tent->name.s;
-		LIST_INSERT_HEAD(head, re, list);
-	}
-	return 0;
-}
-
-int re_entries_match(FILE *f, struct re_entry_head *head, const char *s, int *match)
-{
-	struct re_entry *re;
+	struct tent_tree *tt;
 	int len = strlen(s);
 
 	*match = 0;
-	LIST_FOREACH(re, head, list) {
+	RB_FOREACH(tt, tent_entries, exclusion_root) {
 		int rc;
-		rc = pcre_exec(re->re, NULL, s, len, 0, 0, NULL, 0);
+		rc = pcre_exec(tt->tent->re, NULL, s, len, 0, 0, NULL, 0);
 		if(rc == 0) {
 			*match = 1;
 			if(do_verbose) {
-				fprintf(f, "tup info: Ignoring file '%s' because it matched the regex '%s'\n", s, re->s);
+				fprintf(f, "tup info: Ignoring file '%s' because it matched the regex '%s'\n", s, tt->tent->name.s);
 			}
 			break;
 		} else if(rc != PCRE_ERROR_NOMATCH) {
-			fprintf(f, "tup error: Regex failed to execute: %s\n", re->s);
+			fprintf(f, "tup error: Regex failed to execute: %s\n", tt->tent->name.s);
 			return -1;
 		}
 	}
 	return 0;
 }
-
-void free_re_list(struct re_entry_head *head)
-{
-	while(!LIST_EMPTY(head)) {
-		struct re_entry *re = LIST_FIRST(head);
-		LIST_REMOVE(re, list);
-		free(re);
-	}
-}
