@@ -2,7 +2,7 @@
  *
  * tup - A file-based build system
  *
- * Copyright (C) 2008-2018  Mike Shal <marfey@gmail.com>
+ * Copyright (C) 2008-2020  Mike Shal <marfey@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2 as
@@ -34,19 +34,17 @@
 #include <sys/stat.h>
 
 static struct file_entry *new_entry(const char *filename);
-static void check_unlink_list(const struct pel_group *pg,
-			      struct file_entry_head *u_head);
+static void check_unlink_list(const char *filename, struct file_entry_head *u_head);
 static void handle_unlink(struct file_info *info);
 static int update_write_info(FILE *f, tupid_t cmdid, struct file_info *info,
-			     int *warnings, struct tup_entry_head *entryhead,
-			     enum check_type_t check_only);
+			     int *warnings, enum check_type_t check_only);
 static int update_read_info(FILE *f, tupid_t cmdid, struct file_info *info,
-			    struct tup_entry_head *entryhead,
 			    int full_deps, tupid_t vardt,
 			    int *important_link_removed);
-static int add_config_files_locked(struct file_info *finfo, struct tup_entry *tent);
+static int add_config_files_locked(struct file_info *finfo, struct tup_entry *tent, int full_deps);
 static int add_parser_files_locked(struct file_info *finfo,
-				   struct tupid_entries *root, tupid_t vardt);
+				   struct tent_entries *root, tupid_t vardt,
+				   int full_deps);
 
 int init_file_info(struct file_info *info, const char *variant_dir, int do_unlink)
 {
@@ -61,7 +59,7 @@ int init_file_info(struct file_info *info, const char *variant_dir, int do_unlin
 	RB_INIT(&info->group_sticky_root);
 	RB_INIT(&info->used_groups_root);
 	RB_INIT(&info->output_root);
-	LIST_INIT(&info->exclusion_list);
+	RB_INIT(&info->exclusion_root);
 	pthread_mutex_init(&info->lock, NULL);
 	pthread_cond_init(&info->cond, NULL);
 	/* Root variant gets a NULL variant_dir so we can skip trying to do the
@@ -80,12 +78,12 @@ int init_file_info(struct file_info *info, const char *variant_dir, int do_unlin
 
 void cleanup_file_info(struct file_info *info)
 {
-	free_re_list(&info->exclusion_list);
-	free_tupid_tree(&info->output_root);
-	free_tupid_tree(&info->used_groups_root);
-	free_tupid_tree(&info->group_sticky_root);
-	free_tupid_tree(&info->normal_root);
-	free_tupid_tree(&info->sticky_root);
+	free_tent_tree(&info->exclusion_root);
+	free_tent_tree(&info->output_root);
+	free_tent_tree(&info->used_groups_root);
+	free_tent_tree(&info->group_sticky_root);
+	free_tent_tree(&info->normal_root);
+	free_tent_tree(&info->sticky_root);
 }
 
 void finfo_lock(struct file_info *info)
@@ -158,7 +156,7 @@ int handle_open_file(enum access_type at, const char *filename,
 			LIST_INSERT_HEAD(&info->read_list, fent, list);
 			break;
 		case ACCESS_WRITE:
-			check_unlink_list(&fent->pg, &info->unlink_list);
+			check_unlink_list(filename, &info->unlink_list);
 			LIST_INSERT_HEAD(&info->write_list, fent, list);
 			break;
 		case ACCESS_UNLINK:
@@ -182,7 +180,6 @@ int write_files(FILE *f, tupid_t cmdid, struct file_info *info, int *warnings,
 		int full_deps, tupid_t vardt,
 		int *important_link_removed)
 {
-	struct tup_entry_head *entrylist;
 	struct tmpdir *tmpdir;
 	int tmpdir_bork = 0;
 	int rc1 = 0, rc2 = 0;
@@ -195,7 +192,7 @@ int write_files(FILE *f, tupid_t cmdid, struct file_info *info, int *warnings,
 			int match = 0;
 
 			tmpdir = LIST_FIRST(&info->tmpdir_list);
-			if(re_entries_match(f, &info->exclusion_list, tmpdir->dirname, &match) < 0)
+			if(exclusion_match(f, &info->exclusion_root, tmpdir->dirname, &match) < 0)
 				return -1;
 			if(match) {
 				if(mkdir(tmpdir->dirname, 0777) < 0) {
@@ -219,9 +216,7 @@ int write_files(FILE *f, tupid_t cmdid, struct file_info *info, int *warnings,
 		}
 	}
 
-	entrylist = tup_entry_get_list();
-	rc1 = update_write_info(f, cmdid, info, warnings, entrylist, check_only);
-	tup_entry_release_list();
+	rc1 = update_write_info(f, cmdid, info, warnings, check_only);
 
 	/* Only process file inputs if the command wasn't signaled. We
 	 * obviously need to check them if the command succeeded to ensure DAG
@@ -233,9 +228,7 @@ int write_files(FILE *f, tupid_t cmdid, struct file_info *info, int *warnings,
 	 * back to the commandline ASAP.
 	 */
 	if(check_only != CHECK_SIGNALLED) {
-		entrylist = tup_entry_get_list();
-		rc2 = update_read_info(f, cmdid, info, entrylist, full_deps, vardt, important_link_removed);
-		tup_entry_release_list();
+		rc2 = update_read_info(f, cmdid, info, full_deps, vardt, important_link_removed);
 	}
 	finfo_unlock(info);
 
@@ -265,86 +258,74 @@ int write_files(FILE *f, tupid_t cmdid, struct file_info *info, int *warnings,
 	return -1;
 }
 
-int add_config_files(struct file_info *finfo, struct tup_entry *tent)
+int add_config_files(struct file_info *finfo, struct tup_entry *tent, int full_deps)
 {
 	int rc;
 	finfo_lock(finfo);
-	rc = add_config_files_locked(finfo, tent);
+	rc = add_config_files_locked(finfo, tent, full_deps);
 	finfo_unlock(finfo);
 	return rc;
 }
 
-int add_parser_files(struct file_info *finfo, struct tupid_entries *root, tupid_t vardt)
+int add_parser_files(struct file_info *finfo, struct tent_entries *root,
+		     tupid_t vardt, int full_deps)
 {
 	int rc;
 	finfo_lock(finfo);
-	rc = add_parser_files_locked(finfo, root, vardt);
+	rc = add_parser_files_locked(finfo, root, vardt, full_deps);
 	finfo_unlock(finfo);
 	return rc;
 }
 
-/* Ghost directories in the /-tree have mtimes set to zero if they exist. This way we can
- * distinguish between a directory being created where there wasn't one previously (t4064).
- */
-static int set_directories_to_zero(tupid_t dt, tupid_t slash)
-{
-	struct tup_entry *tent;
-
-	if(dt == slash)
-		return 0;
-	if(tup_entry_add(dt, &tent) < 0)
-		return -1;
-
-	/* Short circuit if we found a dir that is already set */
-	if(tent->mtime == 0)
-		return 0;
-
-	if(tup_db_set_mtime(tent, 0) < 0)
-		return -1;
-	return set_directories_to_zero(tent->dt, slash);
-}
-
-static int add_node_to_list(tupid_t dt, struct pel_group *pg,
-			    struct tup_entry_head *head, int full_deps, const char *full_path)
+static int add_node_to_tree(tupid_t dt, const char *filename,
+			    struct tent_entries *root, int full_deps)
 {
 	tupid_t new_dt;
 	struct path_element *pel = NULL;
+	struct pel_group pg;
 	struct tup_entry *tent;
+	struct tup_entry *new_dtent;
 
-	new_dt = find_dir_tupid_dt_pg(dt, pg, &pel, 1, full_deps);
+	if(get_path_elements(filename, &pg) < 0)
+		return -1;
+
+	new_dt = find_dir_tupid_dt_pg(dt, &pg, &pel, 1, full_deps);
 	if(new_dt < 0)
 		return -1;
 	if(new_dt == 0) {
+		del_pel_group(&pg);
 		return 0;
 	}
 	if(pel == NULL) {
 		/* This can happen for the '.' entry */
+		del_pel_group(&pg);
 		return 0;
 	}
 
-	if(tup_db_select_tent_part(new_dt, pel->path, pel->len, &tent) < 0)
+	if(tup_entry_add(new_dt, &new_dtent) < 0)
+		return -1;
+	if(tup_db_select_tent_part(new_dtent, pel->path, pel->len, &tent) < 0)
 		return -1;
 	if(!tent) {
 		time_t mtime = -1;
-		if(full_deps && (pg->pg_flags & PG_OUTSIDE_TUP)) {
-			struct stat buf;
-			if(lstat(full_path, &buf) == 0) {
-				mtime = MTIME(buf);
-			}
-			if(set_directories_to_zero(new_dt, slash_dt()) < 0)
+		int type = TUP_NODE_GHOST;
+		if(full_deps && (pg.pg_flags & PG_OUTSIDE_TUP)) {
+			if(get_outside_tup_mtime(new_dtent, pel, &mtime) < 0)
 				return -1;
 		}
-		/* Note that full-path entries are always ghosts since we don't scan them. They
-		 * can still have a valid mtime, though.
+		/* Note that full-path entries are always ghosts since we don't
+		 * scan them. Files can still have a valid mtime, and
+		 * directories have an mtime of 0, though.
 		 */
-		if(tup_db_node_insert_tent(new_dt, pel->path, pel->len, TUP_NODE_GHOST, mtime, -1, &tent) < 0) {
+		if(tup_db_node_insert_tent(new_dtent, pel->path, pel->len, type, mtime, -1, &tent) < 0) {
 			fprintf(stderr, "tup error: Node '%.*s' doesn't exist in directory %lli, and no luck creating a ghost node there.\n", pel->len, pel->path, new_dt);
 			return -1;
 		}
 	}
 	free(pel);
+	del_pel_group(&pg);
 
-	if(tent->type == TUP_NODE_DIR || tent->type == TUP_NODE_GENERATED_DIR) {
+	if(tent->type == TUP_NODE_DIR || tent->type == TUP_NODE_GENERATED_DIR || tent->mtime == 0) {
 		/* We don't track dependencies on directory nodes for commands. Note that
 		 * some directory accesses may create ghost nodes as placeholders for the
 		 * directory until a real directory is created there (eg: t5077). In this
@@ -354,12 +335,15 @@ static int add_node_to_list(tupid_t dt, struct pel_group *pg,
 		 */
 		return 0;
 	}
-	tup_entry_list_add(tent, head);
+	if(tent_tree_add_dup(root, tent) < 0) {
+		return -1;
+	}
 
 	return 0;
 }
 
 #ifdef _WIN32
+#include <windows.h>
 #define WINDOWS_TICK 10000000
 #define SEC_TO_UNIX_EPOCH 11644473600LL
 
@@ -374,19 +358,39 @@ static int file_set_mtime(struct tup_entry *tent, const char *file)
 	WIN32_FILE_ATTRIBUTE_DATA data;
 	wchar_t widefile[WIDE_PATH_MAX];
 	int prefix_len = 4;
+	int pos = 0;
 	wchar_t *dest;
 	wchar_t *tmp;
 
 	dest = widefile;
-	if(is_full_path(file)) {
-		/* Everything from the DLL injection is a full path, but we
-		 * need to prefix with \\?\ make GetFileAttributesEx work with
-		 * >260 character paths.
+	/* Everything from the DLL injection is a full path, but we need to
+	 * prefix with \\?\ make GetFileAttributesEx work with >260 character
+	 * paths.
+	 */
+	wcscpy(widefile, L"\\\\?\\");
+	dest += prefix_len;
+	pos += prefix_len;
+	if(!is_full_path(file)) {
+		/* !tup_ln links give us ACCESS_WRITE files with tup-top
+		 * relative paths, so we need to prefix with the working
+		 * directory or else GetFileAttributesExW below can fail since
+		 * we don't have the dir_mutex.
 		 */
-		wcscpy(widefile, L"\\\\?\\");
-		dest += prefix_len;
+		if(file[0] != '.' && file[1] != '/') {
+			fprintf(stderr, "tup internal error: Expected relative path to start with ./ in file_set_mtime(): %s\n", file);
+			return -1;
+		}
+		MultiByteToWideChar(CP_UTF8, 0, get_tup_top(), -1, dest, WIDE_PATH_MAX - pos);
+		dest += get_tup_top_len();
+		pos += get_tup_top_len();
+		if(pos >= WIDE_PATH_MAX) {
+			fprintf(stderr, "tup error: WIDE_PATH_MAX is too small in file_set_mtime()\n");
+			return -1;
+		}
+		/* Skip the '.' part of the path */
+		file++;
 	}
-	MultiByteToWideChar(CP_UTF8, 0, file, -1, dest, WIDE_PATH_MAX - prefix_len);
+	MultiByteToWideChar(CP_UTF8, 0, file, -1, dest, WIDE_PATH_MAX - pos);
 	widefile[WIDE_PATH_MAX-1] = 0;
 	/* Backout the forward-slash conversion used for regex matching. */
 	for(tmp=dest; *tmp != 0; tmp++) {
@@ -421,65 +425,60 @@ static int file_set_mtime(struct tup_entry *tent, const char *file)
 }
 #endif
 
-static int add_config_files_locked(struct file_info *finfo, struct tup_entry *tent)
+static int add_config_files_locked(struct file_info *finfo, struct tup_entry *tent, int full_deps)
 {
 	struct file_entry *r;
-	struct tup_entry_head *entrylist;
-	int full_deps = tup_option_get_int("updater.full_deps");
+	struct tent_entries root = {NULL};
 
-	entrylist = tup_entry_get_list();
 	while(!LIST_EMPTY(&finfo->read_list)) {
-		struct tup_entry *tmp;
 		r = LIST_FIRST(&finfo->read_list);
 
-		if(add_node_to_list(DOT_DT, &r->pg, entrylist, full_deps, r->filename) < 0)
+		if(add_node_to_tree(DOT_DT, r->filename, &root, full_deps) < 0)
 			return -1;
 
-		/* Don't link to ourself */
-		tmp = LIST_FIRST(entrylist);
-		if(tmp == tent) {
-			tup_entry_list_del(tmp);
-		}
-
 		del_file_entry(r);
 	}
-	if(tup_db_check_config_inputs(tent, entrylist) < 0)
+
+	/* Don't link to ourself */
+	tent_tree_remove(&root, tent);
+
+	if(tup_db_check_config_inputs(tent, &root) < 0)
 		return -1;
-	tup_entry_release_list();
+	free_tent_tree(&root);
 
 	return 0;
 }
 
 static int add_parser_files_locked(struct file_info *finfo,
-				   struct tupid_entries *root, tupid_t vardt)
+				   struct tent_entries *root, tupid_t vardt,
+				   int full_deps)
 {
 	struct file_entry *r;
 	struct mapping *map;
-	struct tup_entry_head *entrylist;
 	struct tup_entry *tent;
+	struct tent_entries tmproot = {NULL};
+	struct tent_tree *tt;
 	int map_bork = 0;
-	int full_deps = tup_option_get_int("updater.full_deps");
 
-	entrylist = tup_entry_get_list();
 	while(!LIST_EMPTY(&finfo->read_list)) {
 		r = LIST_FIRST(&finfo->read_list);
-		if(add_node_to_list(DOT_DT, &r->pg, entrylist, full_deps, r->filename) < 0)
+		if(add_node_to_tree(DOT_DT, r->filename, &tmproot, full_deps) < 0)
 			return -1;
 		del_file_entry(r);
 	}
 	while(!LIST_EMPTY(&finfo->var_list)) {
 		r = LIST_FIRST(&finfo->var_list);
 
-		if(add_node_to_list(vardt, &r->pg, entrylist, 0, NULL) < 0)
+		if(add_node_to_tree(vardt, r->filename, &tmproot, 0) < 0)
 			return -1;
 		del_file_entry(r);
 	}
-	LIST_FOREACH(tent, entrylist, list) {
-		if(strcmp(tent->name.s, ".gitignore") != 0)
-			if(tupid_tree_add_dup(root, tent->tnode.tupid) < 0)
+	RB_FOREACH(tt, tent_entries, &tmproot) {
+		if(strcmp(tt->tent->name.s, ".gitignore") != 0)
+			if(tent_tree_add_dup(root, tt->tent) < 0)
 				return -1;
 	}
-	tup_entry_release_list();
+	free_tent_tree(&tmproot);
 
 	/* TODO: write_list not needed here? */
 	while(!LIST_EMPTY(&finfo->write_list)) {
@@ -519,19 +518,12 @@ static struct file_entry *new_entry(const char *filename)
 		free(fent);
 		return NULL;
 	}
-
-	if(get_path_elements(fent->filename, &fent->pg) < 0) {
-		free(fent->filename);
-		free(fent);
-		return NULL;
-	}
 	return fent;
 }
 
 void del_file_entry(struct file_entry *fent)
 {
 	LIST_REMOVE(fent, list);
-	del_pel_group(&fent->pg);
 	free(fent->filename);
 	free(fent);
 }
@@ -539,17 +531,9 @@ void del_file_entry(struct file_entry *fent)
 int handle_rename(const char *from, const char *to, struct file_info *info)
 {
 	struct file_entry *fent;
-	struct pel_group pg_from;
-	struct pel_group pg_to;
-
-	if(get_path_elements(from, &pg_from) < 0)
-		return -1;
-	if(get_path_elements(to, &pg_to) < 0)
-		return -1;
 
 	LIST_FOREACH(fent, &info->write_list, list) {
-		if(pg_eq(&fent->pg, &pg_from)) {
-			del_pel_group(&fent->pg);
+		if(name_cmp(fent->filename, from) == 0) {
 			free(fent->filename);
 
 			fent->filename = strdup(to);
@@ -557,13 +541,10 @@ int handle_rename(const char *from, const char *to, struct file_info *info)
 				perror("strdup");
 				return -1;
 			}
-			if(get_path_elements(fent->filename, &fent->pg) < 0)
-				return -1;
 		}
 	}
 	LIST_FOREACH(fent, &info->read_list, list) {
-		if(pg_eq(&fent->pg, &pg_from)) {
-			del_pel_group(&fent->pg);
+		if(name_cmp(fent->filename, from) == 0) {
 			free(fent->filename);
 
 			fent->filename = strdup(to);
@@ -571,14 +552,10 @@ int handle_rename(const char *from, const char *to, struct file_info *info)
 				perror("strdup");
 				return -1;
 			}
-			if(get_path_elements(fent->filename, &fent->pg) < 0)
-				return -1;
 		}
 	}
 
-	check_unlink_list(&pg_to, &info->unlink_list);
-	del_pel_group(&pg_to);
-	del_pel_group(&pg_from);
+	check_unlink_list(to, &info->unlink_list);
 	return 0;
 }
 
@@ -590,13 +567,12 @@ void del_map(struct mapping *map)
 	free(map);
 }
 
-static void check_unlink_list(const struct pel_group *pg,
-			      struct file_entry_head *u_head)
+static void check_unlink_list(const char *filename, struct file_entry_head *u_head)
 {
 	struct file_entry *fent, *tmp;
 
 	LIST_FOREACH_SAFE(fent, u_head, list, tmp) {
-		if(pg_eq(&fent->pg, pg)) {
+		if(name_cmp(filename, fent->filename) == 0) {
 			del_file_entry(fent);
 		}
 	}
@@ -610,12 +586,12 @@ static void handle_unlink(struct file_info *info)
 		u = LIST_FIRST(&info->unlink_list);
 
 		LIST_FOREACH_SAFE(fent, &info->write_list, list, tmp) {
-			if(pg_eq(&fent->pg, &u->pg)) {
+			if(name_cmp(fent->filename, u->filename) == 0) {
 				del_file_entry(fent);
 			}
 		}
 		LIST_FOREACH_SAFE(fent, &info->read_list, list, tmp) {
-			if(pg_eq(&fent->pg, &u->pg)) {
+			if(name_cmp(fent->filename, u->filename) == 0) {
 				del_file_entry(fent);
 			}
 		}
@@ -628,9 +604,10 @@ static int create_ignored_file(FILE *f, struct file_entry *w)
 {
 	struct path_element *pel = NULL;
 	struct tup_entry *tent;
+	struct tup_entry *dtent;
 	tupid_t dt;
 
-	dt = find_dir_tupid_dt_pg(DOT_DT, &w->pg, &pel, SOTGV_IGNORE_DIRS, 0);
+	dt = find_dir_tupid_dt(DOT_DT, w->filename, &pel, SOTGV_IGNORE_DIRS, 0);
 	if(dt < 0) {
 		fprintf(f, "tup error: Unable to create directory tree for ignored file: %s\n", w->filename);
 		return -1;
@@ -639,7 +616,9 @@ static int create_ignored_file(FILE *f, struct file_entry *w)
 		fprintf(f, "tup internal error: create_ignored_file() didn't get a final pel pointer for file: %s\n", w->filename);
 		return -1;
 	}
-	tent = tup_db_create_node_part(dt, pel->path, pel->len, TUP_NODE_FILE, -1, NULL);
+	if(tup_entry_add(dt, &dtent) < 0)
+		return -1;
+	tent = tup_db_create_node_part(dtent, pel->path, pel->len, TUP_NODE_FILE, -1, NULL);
 	if(!tent)
 		return -1;
 	free(pel);
@@ -647,23 +626,24 @@ static int create_ignored_file(FILE *f, struct file_entry *w)
 }
 
 static int update_write_info(FILE *f, tupid_t cmdid, struct file_info *info,
-			     int *warnings, struct tup_entry_head *entryhead,
-			     enum check_type_t check_only)
+			     int *warnings, enum check_type_t check_only)
 {
 	struct file_entry *w;
 	struct file_entry *r;
 	struct file_entry *tmp;
 	struct tup_entry *tent;
+	struct tent_entries root = {NULL};
 	int write_bork = 0;
 
 	while(!LIST_EMPTY(&info->write_list)) {
 		tupid_t newdt;
 		struct path_element *pel = NULL;
+		struct pel_group pg;
 		int match = 0;
 
 		w = LIST_FIRST(&info->write_list);
 
-		if(re_entries_match(f, &info->exclusion_list, w->filename, &match) < 0)
+		if(exclusion_match(f, &info->exclusion_root, w->filename, &match) < 0)
 			return -1;
 		if(match) {
 			if(create_ignored_file(f, w) < 0)
@@ -673,28 +653,35 @@ static int update_write_info(FILE *f, tupid_t cmdid, struct file_info *info,
 
 		/* Remove duplicate write entries */
 		LIST_FOREACH_SAFE(r, &info->write_list, list, tmp) {
-			if(r != w && pg_eq(&w->pg, &r->pg)) {
+			if(r != w && (name_cmp(w->filename, r->filename) == 0)) {
 				del_file_entry(r);
 			}
 		}
 
-		if(w->pg.pg_flags & PG_HIDDEN) {
+		if(get_path_elements(w->filename, &pg) < 0)
+			return -1;
+		if(pg.pg_flags & PG_HIDDEN) {
 			if(warnings) {
 				fprintf(f, "tup warning: Writing to hidden file '%s'\n", w->filename);
 				(*warnings)++;
 			}
+			del_pel_group(&pg);
 			goto out_skip;
 		}
 
 		tent = NULL;
-		newdt = find_dir_tupid_dt_pg(DOT_DT, &w->pg, &pel, 0, 0);
+		newdt = find_dir_tupid_dt_pg(DOT_DT, &pg, &pel, 0, 0);
+		del_pel_group(&pg);
 		if(newdt > 0) {
+			struct tup_entry *dtent;
+			if(tup_entry_add(newdt, &dtent) < 0)
+				return -1;
 			if(!pel) {
 				fprintf(f, "tup internal error: find_dir_tupid_dt_pg() in write_files() didn't get a final pel pointer for file: %s\n", w->filename);
 				return -1;
 			}
 
-			if(tup_db_select_tent_part(newdt, pel->path, pel->len, &tent) < 0)
+			if(tup_db_select_tent_part(dtent, pel->path, pel->len, &tent) < 0)
 				return -1;
 			free(pel);
 		}
@@ -717,7 +704,9 @@ static int update_write_info(FILE *f, tupid_t cmdid, struct file_info *info,
 		} else {
 			struct mapping *map;
 			int mapping_set = 0;
-			tup_entry_list_add(tent, entryhead);
+
+			if(tent_tree_add_dup(&root, tent) < 0)
+				return -1;
 
 			LIST_FOREACH(map, &info->mapping_list, list) {
 				if(strcmp(map->realname, w->filename) == 0) {
@@ -750,7 +739,7 @@ out_skip:
 		del_file_entry(w);
 	}
 
-	if(tup_db_check_actual_outputs(f, cmdid, entryhead, &info->output_root, &info->mapping_list, &write_bork, info->do_unlink, check_only==CHECK_SUCCESS) < 0)
+	if(tup_db_check_actual_outputs(f, cmdid, &root, &info->output_root, &info->mapping_list, &write_bork, info->do_unlink, check_only==CHECK_SUCCESS) < 0)
 		return -1;
 
 	while(!LIST_EMPTY(&info->mapping_list)) {
@@ -774,6 +763,7 @@ out_skip:
 		del_map(map);
 	}
 
+	free_tent_tree(&root);
 	if(write_bork)
 		return -1;
 
@@ -781,21 +771,20 @@ out_skip:
 }
 
 static int update_read_info(FILE *f, tupid_t cmdid, struct file_info *info,
-			    struct tup_entry_head *entryhead,
 			    int full_deps, tupid_t vardt,
 			    int *important_link_removed)
 {
 	struct file_entry *r;
-	struct tupid_tree *tt;
+	struct tent_entries root = {NULL};
 
 	while(!LIST_EMPTY(&info->read_list)) {
 		int match = 0;
 		r = LIST_FIRST(&info->read_list);
 
-		if(re_entries_match(f, &info->exclusion_list, r->filename, &match) < 0)
+		if(exclusion_match(f, &info->exclusion_root, r->filename, &match) < 0)
 			return -1;
 		if(!match) {
-			if(add_node_to_list(DOT_DT, &r->pg, entryhead, full_deps, r->filename) < 0)
+			if(add_node_to_tree(DOT_DT, r->filename, &root, full_deps) < 0)
 				return -1;
 		}
 		del_file_entry(r);
@@ -804,19 +793,15 @@ static int update_read_info(FILE *f, tupid_t cmdid, struct file_info *info,
 	while(!LIST_EMPTY(&info->var_list)) {
 		r = LIST_FIRST(&info->var_list);
 
-		if(add_node_to_list(vardt, &r->pg, entryhead, 0, NULL) < 0)
+		if(add_node_to_tree(vardt, r->filename, &root, 0) < 0)
 			return -1;
 		del_file_entry(r);
 	}
 
-	RB_FOREACH(tt, tupid_entries, &info->used_groups_root) {
-		struct tup_entry *tent;
-		if(tup_entry_add(tt->tupid, &tent) < 0)
-			return -1;
-		tup_entry_list_add(tent, entryhead);
-	}
+	tent_tree_copy(&root, &info->used_groups_root);
 
-	if(tup_db_check_actual_inputs(f, cmdid, entryhead, &info->sticky_root, &info->normal_root, &info->group_sticky_root, &info->output_root, important_link_removed) < 0)
+	if(tup_db_check_actual_inputs(f, cmdid, &root, &info->sticky_root, &info->normal_root, &info->group_sticky_root, &info->output_root, important_link_removed) < 0)
 		return -1;
+	free_tent_tree(&root);
 	return 0;
 }
