@@ -2,7 +2,7 @@
  *
  * tup - A file-based build system
  *
- * Copyright (C) 2008-2018  Mike Shal <marfey@gmail.com>
+ * Copyright (C) 2008-2020  Mike Shal <marfey@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2 as
@@ -31,6 +31,8 @@ struct tup_entry_head;
 struct path_element;
 struct pel_group;
 
+#define EXTERNAL_DIRECTORY_MTIME 0
+
 #define SOTGV_CREATE_DIRS 2
 #define SOTGV_IGNORE_DIRS 3
 
@@ -51,6 +53,7 @@ tupid_t find_dir_tupid_dt(tupid_t dt, const char *dir,
 			  struct path_element **last, int sotgv, int full_deps);
 tupid_t find_dir_tupid_dt_pg(tupid_t dt, struct pel_group *pg,
 			     struct path_element **last, int sotgv, int full_deps);
+int get_outside_tup_mtime(struct tup_entry *parent, struct path_element *pel, time_t *mtime);
 int gimme_tent(const char *name, struct tup_entry **entry);
 
 int delete_file(struct tup_entry *tent);
