@@ -2,7 +2,7 @@
  *
  * tup - A file-based build system
  *
- * Copyright (C) 2008-2018  Mike Shal <marfey@gmail.com>
+ * Copyright (C) 2008-2020  Mike Shal <marfey@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2 as
@@ -26,6 +26,7 @@
 #include "db.h"
 #include "container.h"
 #include "compat.h"
+#include "tent_list.h"
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -402,20 +403,20 @@ int add_graph_stickies(struct graph *g)
 
 	TAILQ_FOREACH(n, &g->node_list, list) {
 		if(n->tent->type == TUP_NODE_CMD) {
-			struct tupid_entries sticky_root = {NULL};
-			struct tupid_tree *tt;
+			struct tent_entries sticky_root = {NULL};
+			struct tent_tree *tt;
 			struct node *inputn;
 
 			if(tup_db_get_inputs(n->tent->tnode.tupid, &sticky_root, NULL, NULL) < 0)
 				return -1;
-			RB_FOREACH(tt, tupid_entries, &sticky_root) {
-				inputn = find_node(g, tt->tupid);
+			RB_FOREACH(tt, tent_entries, &sticky_root) {
+				inputn = find_node(g, tt->tent->tnode.tupid);
 				if(inputn) {
 					if(create_edge(inputn, n, TUP_LINK_STICKY) < 0)
 						return -1;
 				}
 			}
-			free_tupid_tree(&sticky_root);
+			free_tent_tree(&sticky_root);
 		}
 	}
 	return 0;
@@ -456,7 +457,7 @@ edge_create:
 	return 0;
 }
 
-int nodes_are_connected(struct tup_entry *src, struct tupid_entries *valid_root,
+int nodes_are_connected(struct tup_entry *src, struct tent_entries *valid_root,
 			int *connected)
 {
 	struct graph g;
@@ -478,7 +479,7 @@ int nodes_are_connected(struct tup_entry *src, struct tupid_entries *valid_root,
 		n = TAILQ_FIRST(&g.plist);
 
 		if(src != n->tent &&
-		   tupid_tree_search(valid_root, n->tent->tnode.tupid) != NULL) {
+		   tent_tree_search(valid_root, n->tent) != NULL) {
 			*connected = 1;
 			goto out_cleanup;
 		}
@@ -585,7 +586,7 @@ static int prune_node(struct graph *g, struct node *n, int *num_pruned, enum gra
 int prune_graph(struct graph *g, int argc, char **argv, int *num_pruned,
 		enum graph_prune_type gpt, int verbose)
 {
-	struct tup_entry_head *prune_list;
+	struct tent_list_head prune_list;
 	struct tupid_entries dir_root = {NULL};
 	int x;
 	int dashdash = 0;
@@ -593,7 +594,7 @@ int prune_graph(struct graph *g, int argc, char **argv, int *num_pruned,
 
 	*num_pruned = 0;
 
-	prune_list = tup_entry_get_list();
+	tent_list_init(&prune_list);
 	for(x=0; x<argc; x++) {
 		struct tup_entry *tent;
 
@@ -621,19 +622,21 @@ int prune_graph(struct graph *g, int argc, char **argv, int *num_pruned,
 			if(tupid_tree_add_dup(&dir_root, tent->tnode.tupid) < 0)
 				return -1;
 		} else {
-			tup_entry_list_add(tent, prune_list);
+			if(tent_list_add_head(&prune_list, tent) < 0)
+				return -1;
 		}
 	}
 
 	if(do_prune) {
-		struct tup_entry *tent;
+		struct tent_list *tl;
 		struct node *n;
 		struct node *tmp;
 
 		/* For explicit files: Just see if we have the node in the
 		 * PDAG, and if so, mark it.
 		 */
-		LIST_FOREACH(tent, prune_list, list) {
+		tent_list_foreach(tl, &prune_list) {
+			struct tup_entry *tent = tl->tent;
 			n = find_node(g, tent->tnode.tupid);
 			if(n) {
 				mark_nodes(n);
@@ -665,11 +668,11 @@ int prune_graph(struct graph *g, int argc, char **argv, int *num_pruned,
 		}
 	}
 	free_tupid_tree(&dir_root);
-	tup_entry_release_list();
+	free_tent_list(&prune_list);
 	return 0;
 
 out_err:
-	tup_entry_release_list();
+	free_tent_list(&prune_list);
 	return -1;
 }
 
