@@ -2,7 +2,7 @@
  *
  * tup - A file-based build system
  *
- * Copyright (C) 2009-2018  Mike Shal <marfey@gmail.com>
+ * Copyright (C) 2009-2020  Mike Shal <marfey@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2 as
@@ -37,7 +37,7 @@
 #include <unistd.h>
 #include <sys/stat.h>
 
-int tup_init(void)
+int tup_init(int argc, char **argv)
 {
 	if(find_tup_dir() != 0) {
 		fprintf(stderr, "tup %s usage: tup [args]\n", tup_version);
@@ -45,7 +45,7 @@ int tup_init(void)
 		fprintf(stderr, "No .tup directory found. Either create a Tupfile.ini file at the top of your project, or manually run 'tup init' there.\n");
 		return -1;
 	}
-	if(tup_entry_init() < 0) {
+	if(tup_option_init(argc, argv) < 0) {
 		return -1;
 	}
 	if(server_pre_init() < 0) {
@@ -201,7 +201,7 @@ int init_command(int argc, char **argv)
 			fprintf(stderr, "tup warning: database already exists in directory: %s\n", wd);
 		}
 		close(fd);
-		return 0;
+		return 1;
 	}
 
 	if(fchdir(fd) < 0) {
