@@ -2,7 +2,7 @@
  *
  * tup - A file-based build system
  *
- * Copyright (C) 2008-2018  Mike Shal <marfey@gmail.com>
+ * Copyright (C) 2008-2020  Mike Shal <marfey@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2 as
@@ -43,7 +43,7 @@ struct parser_server;
 struct lua_State;
 
 struct tupfile {
-	tupid_t tupid;
+	struct tup_entry *tent;
 	struct variant *variant;
 	struct tup_entry *curtent;
 	struct tup_entry *srctent;
@@ -54,9 +54,9 @@ struct tupfile {
 	struct vardb vdb;
 	struct node_vardb node_db;
 	struct tupid_entries cmd_root;
-	struct tupid_entries env_root;
+	struct tent_entries env_root;
 	struct string_entries bang_root;
-	struct tupid_entries input_root;
+	struct tent_entries input_root;
 	struct tupid_entries directory_root;
 	struct tupid_entries refactoring_cmd_delete_root;
 	FILE *f;
@@ -67,6 +67,7 @@ struct tupfile {
 	struct lua_State *ls;
 	int luaerror;
 	int use_server;
+	int full_deps;
 };
 
 #define MAX_GLOBS 10
@@ -161,8 +162,7 @@ void move_name_list_entry(struct name_list *newnl, struct name_list *oldnl,
 			  struct name_list_entry *nle);
 void delete_name_list(struct name_list *nl);
 int get_name_list(struct tupfile *tf, struct path_list_head *plist,
-		  struct name_list *nl);
-void make_name_list_unique(struct name_list *nl);
+		  struct name_list *nl, int is_variant_copy);
 void init_rule(struct rule *r);
 int execute_rule(struct tupfile *tf, struct rule *r, struct name_list *output_nl);
 int parser_include_file(struct tupfile *tf, const char *file);
@@ -173,7 +173,7 @@ struct graph;
 struct timespan;
 
 void parser_debug_run(void);
-int parse(struct node *n, struct graph *g, struct timespan *ts, int refactoring, int use_server);
+int parse(struct node *n, struct graph *g, struct timespan *ts, int refactoring, int use_server, int full_deps);
 char *eval(struct tupfile *tf, const char *string, int allow_nodes);
 
 #endif
