@@ -2,7 +2,7 @@
  *
  * tup - A file-based build system
  *
- * Copyright (C) 2012-2018  Mike Shal <marfey@gmail.com>
+ * Copyright (C) 2012-2020  Mike Shal <marfey@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2 as
@@ -295,7 +295,9 @@ static int context_check(void)
 
 static int ignore_file(const char *path)
 {
-	if(strncmp(path, "/proc/", 6) == 0)
+	if(strncmp(path, "/dev", 4) == 0)
+		return 1;
+	if(strncmp(path, "/proc", 5) == 0)
 		return 1;
 	if(is_ccache_path(path))
 		return 1;
@@ -1127,7 +1129,7 @@ static int tup_fs_truncate(const char *path, off_t size)
 			put_finfo(finfo);
 			return rc;
 		} else {
-			if(re_entries_match(stderr, &finfo->exclusion_list, peeled, &match) < 0) {
+			if(exclusion_match(stderr, &finfo->exclusion_root, peeled, &match) < 0) {
 				put_finfo(finfo);
 				return -ENOSYS;
 			}
@@ -1168,10 +1170,18 @@ static int tup_fs_utimens(const char *path, const struct timespec ts[2])
 			put_finfo(finfo);
 			return rc;
 		} else {
-			if(re_entries_match(stderr, &finfo->exclusion_list, peeled, &match) < 0) {
+			struct tmpdir *tmpdir;
+			if(exclusion_match(stderr, &finfo->exclusion_root, peeled, &match) < 0) {
 				put_finfo(finfo);
 				return -ENOSYS;
 			}
+			/* Ignore a touch on a temporary directory */
+			LIST_FOREACH(tmpdir, &finfo->tmpdir_list, list) {
+				if(strcmp(tmpdir->dirname, peeled) == 0) {
+					put_finfo(finfo);
+					return 0;
+				}
+			}
 		}
 		put_finfo(finfo);
 	}
