@@ -2,7 +2,7 @@
  *
  * tup - A file-based build system
  *
- * Copyright (C) 2011-2018  Mike Shal <marfey@gmail.com>
+ * Copyright (C) 2011-2020  Mike Shal <marfey@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2 as
@@ -465,7 +465,7 @@ static int exec_internal(struct server *s, const char *cmd, struct tup_env *newe
 	/* dirlen includes the \0, which snprintf does not count. Hence the -1/+1
 	 * adjusting.
 	 */
-	strncpy(dir, get_tup_top() + 1, sizeof(dir));
+	strncpy(dir, get_tup_top() + 1, sizeof(dir) - 1);
 	em.dirlen = get_tup_top_len() - 1;
 	em.dirlen += snprint_tup_entry(dir + em.dirlen,
 				       sizeof(dir) - em.dirlen - 1,
@@ -576,7 +576,7 @@ int server_unlink(void)
 }
 
 int server_run_script(FILE *f, tupid_t tupid, const char *cmdline,
-		      struct tupid_entries *env_root, char **rules)
+		      struct tent_entries *env_root, char **rules)
 {
 	struct tup_entry *tent;
 	struct server s;
