@@ -2,7 +2,7 @@
  *
  * tup - A file-based build system
  *
- * Copyright (C) 2011-2018  Mike Shal <marfey@gmail.com>
+ * Copyright (C) 2011-2020  Mike Shal <marfey@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2 as
@@ -145,7 +145,7 @@ int server_pre_init(void)
 {
 	char c = 0;
 	int rc;
-	full_deps = tup_option_get_int("updater.full_deps");
+	full_deps = tup_option_get_flag("updater.full_deps");
 	if(setpgid(0, 0) < 0) {
 		perror("setpgid");
 		fprintf(stderr, "tup error: Unable to set process group for tup's subprocesses.\n");
