@@ -3,7 +3,7 @@
  * tup - A file-based build system
  *
  * Copyright (C) 2010  James McKaskill
- * Copyright (C) 2010-2018  Mike Shal <marfey@gmail.com>
+ * Copyright (C) 2010-2020  Mike Shal <marfey@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2 as
@@ -32,12 +32,14 @@
 #include <stdio.h>
 #include <fcntl.h>
 #include <errno.h>
+#include <signal.h>
 
 #define TUP_TMP ".tup/tmp"
 
 static int initialize_depfile(struct server *s, char *depfile, HANDLE *h);
 static int process_depfile(struct server *s, HANDLE h);
 static int server_inited = 0;
+static HANDLE nul_handle;
 static BOOL WINAPI console_handler(DWORD cevent);
 static sig_atomic_t event_got = -1;
 
@@ -61,6 +63,7 @@ int server_init(enum server_mode mode)
 	struct flist f = FLIST_INITIALIZER;
 	int cwdlen;
 	wchar_t wwintmpdir[PATH_MAX];
+	SECURITY_ATTRIBUTES sec;
 
 	if(mode) {/* unused */}
 
@@ -70,6 +73,12 @@ int server_init(enum server_mode mode)
 	if(GetModuleFileNameA(NULL, mycwd, PATH_MAX - 1) == 0)
 		return -1;
 
+	memset(&sec, 0, sizeof(sec));
+	sec.nLength = sizeof(sec);
+	sec.lpSecurityDescriptor = NULL;
+	sec.bInheritHandle = TRUE;
+	nul_handle = CreateFile(L"NUL", GENERIC_READ, FILE_SHARE_READ, &sec, OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
+
 	GetTempPath(PATH_MAX, wwintmpdir);
 	WideCharToMultiByte(CP_UTF8, 0, wwintmpdir, -1, wintmpdir, PATH_MAX, NULL, NULL);
 
@@ -145,6 +154,7 @@ int server_init(enum server_mode mode)
 
 int server_quit(void)
 {
+	CloseHandle(nul_handle);
 	return 0;
 }
 
@@ -181,6 +191,7 @@ static int create_process(struct server *s, int dfd, char *cmdline,
 	}
 	swprintf(buf, 64, L".tup\\tmp\\output-%i", s->id);
 	buf[63] = 0;
+	sa.hStdInput = nul_handle;
 	sa.hStdOutput = CreateFile(buf, GENERIC_WRITE, 0, &sec, CREATE_ALWAYS, FILE_ATTRIBUTE_TEMPORARY, NULL);
 	if(sa.hStdOutput == INVALID_HANDLE_VALUE) {
 		fprintf(stderr, "tup error: Unable to create temporary file for stdout\n");
@@ -257,6 +268,7 @@ int server_exec(struct server *s, int dfd, const char *cmd, struct tup_env *newe
 
 	int need_sh = 0;
 	int need_cmd = 0;
+	int append_quote = 0;
 
 	if(dtent) {}
 	if(need_namespacing) {}
@@ -293,14 +305,16 @@ int server_exec(struct server *s, int dfd, const char *cmd, struct tup_env *newe
 			strchr(cmd, '<') != NULL;
 		if(run_in_bash) {
 			strcat(cmdline, BASHSTR);
+			append_quote = 1;
 		} else if(need_sh) {
 			strcat(cmdline, SHSTR);
+			append_quote = 1;
 		} else if(need_cmd) {
 			strcat(cmdline, CMDSTR);
 		}
 	}
 	strcat(cmdline, cmd);
-	if(need_sh) {
+	if(append_quote) {
 		strcat(cmdline, "'");
 	}
 
@@ -451,7 +465,7 @@ int server_parser_stop(struct parser_server *ps)
 }
 
 int server_run_script(FILE *f, tupid_t tupid, const char *cmdline,
-		      struct tupid_entries *env_root, char **rules)
+		      struct tent_entries *env_root, char **rules)
 {
 	if(f || tupid || cmdline || env_root || rules) {/* unsupported */}
 	fprintf(stderr, "tup error: Run scripts are not yet supported on this platform.\n");
@@ -464,8 +478,11 @@ int server_symlink(struct server *s, const char *target, int dfd, const char *li
 	char dest[PATH_MAX];
 	wchar_t wtarget[PATH_MAX];
 	wchar_t wdest[PATH_MAX];
+	int rc;
 
-	dir_mutex_lock(dfd);
+	rc = dir_mutex_lock(dfd);
+	if(rc < 0)
+		return rc;
 	if(snprintf(dest, sizeof(dest), "%s/%s", win32_get_dirpath(dfd), linkpath) >= PATH_MAX) {
 		fprintf(stderr, "tup error: dest path sized too small in symlinkat compat function\n");
 		goto out_err;
