@@ -2,7 +2,7 @@
  *
  * tup - A file-based build system
  *
- * Copyright (C) 2008-2018  Mike Shal <marfey@gmail.com>
+ * Copyright (C) 2008-2020  Mike Shal <marfey@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2 as
@@ -48,6 +48,7 @@
 #include "tup/privs.h"
 #include "tup/flist.h"
 #include "tup/vardb.h"
+#include "tup/variant.h"
 #include "tup/container.h"
 
 static int entry(int argc, char **argv);
@@ -106,11 +107,6 @@ int main(int argc, char **argv)
 		}
 	}
 
-	if(compat_init() < 0) {
-		fprintf(stderr, "tup error: Unable to initialize compatibility lib\n");
-		return -1;
-	}
-
 	argc = argc - cmd_arg;
 	argv = &argv[cmd_arg];
 
@@ -194,9 +190,7 @@ int main(int argc, char **argv)
 		return 0;
 	}
 
-	if(tup_option_init(argc, argv) < 0)
-		return -1;
-	if(tup_init() < 0)
+	if(tup_init(argc, argv) < 0)
 		return 1;
 
 	if(strcmp(cmd, "monitor") == 0) {
@@ -363,8 +357,8 @@ static int inputs(int argc, char **argv)
 	if(tup_db_begin() < 0)
 		return -1;
 	for(x=1; x<argc; x++) {
-		struct tupid_entries inputs = RB_INITIALIZER(&inputs);
-		struct tupid_tree *tt;
+		struct tent_entries inputs = RB_INITIALIZER(&inputs);
+		struct tent_tree *tt;
 		tupid_t cmdid;
 
 		cmdid = strtol(argv[x], NULL, 10);
@@ -374,12 +368,9 @@ static int inputs(int argc, char **argv)
 		}
 		if(tup_db_get_inputs(cmdid, NULL, &inputs, NULL) < 0)
 			return -1;
-		RB_FOREACH(tt, tupid_entries, &inputs) {
-			struct tup_entry *tent;
-			if(tup_entry_add(tt->tupid, &tent) < 0)
-				return -1;
-			if(tent->type != TUP_NODE_GHOST) {
-				print_tup_entry(stdout, tent);
+		RB_FOREACH(tt, tent_entries, &inputs) {
+			if(tt->tent->type != TUP_NODE_GHOST) {
+				print_tup_entry(stdout, tt->tent);
 				printf("\n");
 			}
 		}
@@ -447,10 +438,10 @@ static int graph(int argc, char **argv)
 
 	if(tup_db_begin() < 0)
 		return -1;
-	show_dirs = tup_option_get_int("graph.dirs");
-	show_ghosts = tup_option_get_int("graph.ghosts");
-	show_env = tup_option_get_int("graph.environment");
-	combine = tup_option_get_int("graph.combine");
+	show_dirs = tup_option_get_flag("graph.dirs");
+	show_ghosts = tup_option_get_flag("graph.ghosts");
+	show_env = tup_option_get_flag("graph.environment");
+	combine = tup_option_get_flag("graph.combine");
 
 	if(create_graph(&g, -1, -1) < 0)
 		return -1;
@@ -557,6 +548,7 @@ static int mlink(int argc, char **argv)
 	int x;
 	tupid_t cmdid;
 	struct tup_entry *tent;
+	struct tup_entry *root_tent;
 
 	if(argc < 4) {
 		fprintf(stderr, "Usage: %s cmd -iread_file -owrite_file\n",
@@ -566,7 +558,7 @@ static int mlink(int argc, char **argv)
 
 	if(tup_db_begin() < 0)
 		return -1;
-	if(tup_entry_add(DOT_DT, NULL) < 0)
+	if(tup_entry_add(DOT_DT, &root_tent) < 0)
 		return -1;
 	cmdid = create_command_file(DOT_DT, argv[1], NULL, 0, NULL, 0);
 	if(cmdid < 0) {
@@ -590,7 +582,7 @@ static int mlink(int argc, char **argv)
 			return 1;
 		}
 
-		if(tup_db_select_tent(DOT_DT, name+2, &tent) < 0)
+		if(tup_db_select_tent(root_tent, name+2, &tent) < 0)
 			return -1;
 		if(!tent)
 			return 1;
@@ -723,6 +715,7 @@ static int node_exists(int argc, char **argv)
 {
 	int x;
 	struct tup_entry *tent;
+	struct tup_entry *dtent;
 	tupid_t dt;
 
 	if(tup_db_begin() < 0)
@@ -738,7 +731,9 @@ static int node_exists(int argc, char **argv)
 	argc--;
 	for(x=1; x<argc; x++) {
 		char *p = argv[x];
-		if(tup_db_select_tent(dt, argv[x], &tent) < 0)
+		if(tup_entry_add(dt, &dtent) < 0)
+			return -1;
+		if(tup_db_select_tent(dtent, argv[x], &tent) < 0)
 			return -1;
 		if(!tent) {
 			/* Path replacement is a hack for windows to work. This
@@ -757,7 +752,7 @@ static int node_exists(int argc, char **argv)
 				}
 				p++;
 			}
-			if(tup_db_select_tent(dt, argv[x], &tent) < 0)
+			if(tup_db_select_tent(dtent, argv[x], &tent) < 0)
 				return -1;
 			if(!tent)
 				return -1;
@@ -772,6 +767,8 @@ static int link_exists(int argc, char **argv)
 {
 	struct tup_entry *tenta;
 	struct tup_entry *tentb;
+	struct tup_entry *dtenta;
+	struct tup_entry *dtentb;
 	int exists;
 	int style;
 	tupid_t dta, dtb;
@@ -796,7 +793,9 @@ static int link_exists(int argc, char **argv)
 		return -1;
 	}
 
-	if(tup_db_select_tent(dta, argv[2], &tenta) < 0)
+	if(tup_entry_add(dta, &dtenta) < 0)
+		return -1;
+	if(tup_db_select_tent(dtenta, argv[2], &tenta) < 0)
 		return -1;
 	if(!tenta) {
 		fprintf(stderr, "[31mError: node '%s' doesn't exist.[0m\n", argv[2]);
@@ -809,7 +808,9 @@ static int link_exists(int argc, char **argv)
 		return -1;
 	}
 
-	if(tup_db_select_tent(dtb, argv[4], &tentb) < 0)
+	if(tup_entry_add(dtb, &dtentb) < 0)
+		return -1;
+	if(tup_db_select_tent(dtentb, argv[4], &tentb) < 0)
 		return -1;
 	if(!tentb) {
 		fprintf(stderr, "[31mError: node '%s' doesn't exist.[0m\n", argv[4]);
@@ -841,6 +842,7 @@ static int touch(int argc, char **argv)
 	for(x=1; x<argc; x++) {
 		struct stat buf;
 		struct path_element *pel = NULL;
+		struct tup_entry *dtent;
 		tupid_t dt;
 
 		if(lstat(argv[x], &buf) < 0) {
@@ -857,8 +859,10 @@ static int touch(int argc, char **argv)
 			fprintf(stderr, "Error finding dt for dir '%s' relative to dir %lli\n", argv[x], sub_dir_dt);
 			return -1;
 		}
+		if(tup_entry_add(dt, &dtent) < 0)
+			return -1;
 		if(S_ISDIR(buf.st_mode)) {
-			if(tup_db_create_node(dt, pel->path, TUP_NODE_DIR) == NULL)
+			if(tup_db_create_node(dtent, pel->path, TUP_NODE_DIR) == NULL)
 				return -1;
 		} else if(S_ISREG(buf.st_mode) || S_ISLNK(buf.st_mode)) {
 			if(tup_file_mod_mtime(dt, pel->path, MTIME(buf), 1, 0, NULL) < 0)
@@ -954,16 +958,22 @@ static int varshow_vdb(struct vardb *vdb)
 static int varshow(int argc, char **argv)
 {
 	struct tup_entry *vartent;
+	struct tup_entry *root_tent;
+	struct variant *variant;
 	if(tup_db_begin() < 0)
 		return -1;
-	if(tup_db_select_tent(DOT_DT, TUP_CONFIG, &vartent) < 0)
+	if(tup_entry_add(DOT_DT, &root_tent) < 0)
+		return -1;
+	if(tup_db_select_tent(root_tent, TUP_CONFIG, &vartent) < 0)
+		return -1;
+	if(variant_add(vartent, 1, &variant) < 0)
 		return -1;
 	if(vartent) {
 		if(argc == 1) {
 			struct vardb vdb;
 			if(vardb_init(&vdb) < 0)
 				return -1;
-			if(tup_db_get_vardb(vartent->tnode.tupid, &vdb) < 0)
+			if(tup_db_get_vardb(vartent, &vdb) < 0)
 				return -1;
 			varshow_vdb(&vdb);
 			vardb_close(&vdb);
@@ -971,18 +981,19 @@ static int varshow(int argc, char **argv)
 			int x;
 			struct tup_entry *tent;
 			for(x=1; x<argc; x++) {
-				char *value;
-				if(tup_db_select_tent(vartent->tnode.tupid, argv[x], &tent) < 0)
-					return -1;
+				tent = tup_db_get_var(variant, argv[x], strlen(argv[x]), NULL);
 				if(!tent) {
 					fprintf(stderr, "Unable to find tupid for variable '%s'\n", argv[x]);
 					continue;
 				}
 				if(tent->type == TUP_NODE_VAR) {
-					if(tup_db_get_var_id_alloc(tent->tnode.tupid, &value) < 0)
-						return -1;
-					printf(" - Var[%s] = '%s'\n", argv[x], value);
-					free(value);
+					struct var_entry *ve;
+					ve = vardb_get(&variant->vdb, argv[x], strlen(argv[x]));
+					if(!ve) {
+						fprintf(stderr, "Unable to find vdb entry for variable '%s'\n", argv[x]);
+						continue;
+					}
+					printf(" - Var[%s] = '%s'\n", argv[x], ve->value);
 				} else if(tent->type == TUP_NODE_GHOST) {
 					printf(" - Var[[47;30m%s[0m] is a ghost\n", argv[x]);
 				} else {
@@ -1020,6 +1031,7 @@ static int options(int argc, char **argv)
 static int fake_mtime(int argc, char **argv)
 {
 	struct tup_entry *tent;
+	struct tup_entry *dtent;
 	time_t mtime;
 	tupid_t dt;
 	tupid_t sub_dir_dt;
@@ -1039,7 +1051,9 @@ static int fake_mtime(int argc, char **argv)
 		fprintf(stderr, "tup error: Unable to find dt for node: %s\n", argv[1]);
 		return -1;
 	}
-	if(tup_db_select_tent_part(dt, pel->path, pel->len, &tent) < 0)
+	if(tup_entry_add(dt, &dtent) < 0)
+		return -1;
+	if(tup_db_select_tent_part(dtent, pel->path, pel->len, &tent) < 0)
 		return -1;
 	if(tent == NULL) {
 		fprintf(stderr, "Unable to find node '%.*s' in dir %lli\n", pel->len, pel->path, dt);
