@@ -1,7 +1,7 @@
 #! /bin/sh -e
 # tup - A file-based build system
 #
-# Copyright (C) 2014-2018  Mike Shal <marfey@gmail.com>
+# Copyright (C) 2014-2020  Mike Shal <marfey@gmail.com>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -20,6 +20,7 @@
 
 . ./tup.sh
 check_no_windows run-script
+check_bash
 
 # Use a variant to make sure we don't get double directories
 tmkdir build
@@ -29,7 +30,7 @@ touch sub/foo.txt
 touch bar.txt baz.txt
 
 cat > gen.sh << HERE
-#! /bin/bash
+#! /usr/bin/env bash
 for i in *; do
 	echo ": |> echo \$i |>"
 done
