@@ -1,7 +1,7 @@
 #! /bin/sh -e
 # tup - A file-based build system
 #
-# Copyright (C) 2017-2018  Mike Shal <marfey@gmail.com>
+# Copyright (C) 2017-2020  Mike Shal <marfey@gmail.com>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -23,11 +23,11 @@ check_no_windows paths
 
 tmkdir sub
 cat > sub/Tupfile << HERE
-: input.txt |> !tup_ln |> $PWD/sub/link.txt
+: input.txt |> !tup_ln |> `pwd -P`/sub/link.txt
 HERE
 tup touch sub/input.txt
 update
 
-tup_dep_exist sub "!tup_ln input.txt $PWD/sub/link.txt" sub link.txt
+tup_dep_exist sub "!tup_ln input.txt `pwd -P`/sub/link.txt" sub link.txt
 
 eotup
