@@ -1,7 +1,7 @@
 #! /bin/sh -e
 # tup - A file-based build system
 #
-# Copyright (C) 2010-2018  Mike Shal <marfey@gmail.com>
+# Copyright (C) 2010-2020  Mike Shal <marfey@gmail.com>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -21,11 +21,6 @@
 . ./tup.sh
 check_no_windows shell
 
-if grep '^ID=\(fedora\|arch\)' /etc/os-release > /dev/null; then
-	echo "[33mSkipping on Fedora/Arch because coreutils' mv uses a syscall directly instead of renameat2[0m"
-	eotup
-fi
-
 cat > Tupfile << HERE
 : |> touch .foo; mv .foo bar |> bar
 HERE
