@@ -1,7 +1,7 @@
 #! /bin/sh -e
 # tup - A file-based build system
 #
-# Copyright (C) 2016-2018  Mike Shal <marfey@gmail.com>
+# Copyright (C) 2016-2020  Mike Shal <marfey@gmail.com>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -41,4 +41,18 @@ update
 cmp file.txt build-1/file.txt
 cmp file.txt build-2/file.txt
 
+# Make sure we can re-parse the Tupfile now that we have file.txt in the srcdir
+# and the build dir.
+tup touch Tupfile
+update > .tup/.tupoutput
+if grep 'preserve file.txt' .tup/.tupoutput > /dev/null; then
+	cat .tup/.tupoutput
+	echo "Error: No preserve commands should run when nothing was changed." 1>&2
+	exit 1
+fi
+
+# test that the file contents have been preserved
+cmp file.txt build-1/file.txt
+cmp file.txt build-2/file.txt
+
 eotup
